@@ -1,3 +1,12 @@
+/*                           (C) 2002 C. Barth Netterfield */
+/***************************************************************************
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ ***************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -8,6 +17,7 @@
 #include <math.h>
 
 #include "getdata.h"
+#include "getdata_struct.h"
 
 const char *GD_ERROR_CODES[15] = {"OK",
                             "Could not open Format file",
@@ -26,64 +36,6 @@ const char *GD_ERROR_CODES[15] = {"OK",
                             "Too many levels of recursion"
 };
 
-#define FIELD_LENGTH 16
-#define MAX_FILENAME_LENGTH 180
-#define MAX_LINE_LENGTH 180
-#define MAX_LINCOM 3
-
-struct RawEntryType {
-  char field[FIELD_LENGTH+1];
-  int fp;
-  char type;
-  int size;
-  int samples_per_frame;
-};
-
-struct LincomEntryType {
-  char field[FIELD_LENGTH+1];
-  int n_infields;
-  char in_fields[MAX_LINCOM][FIELD_LENGTH+1];
-  double m[MAX_LINCOM];
-  double b[MAX_LINCOM];
-};
-
-struct LinterpEntryType {
-  char field[FIELD_LENGTH+1];
-  char raw_field[FIELD_LENGTH+1];
-  char linterp_file[MAX_FILENAME_LENGTH];
-  int n_interp;
-  double *x;
-  double *y;
-};
-
-struct MplexEntryType {
-  char field[FIELD_LENGTH+1];
-  char cnt_field[FIELD_LENGTH+1];
-  char data_field[FIELD_LENGTH+1];
-  int i;
-  int max_i;
-};
-
-struct BitEntryType {
-  char field[FIELD_LENGTH+1];
-  char raw_field[FIELD_LENGTH+1];
-  int bitnum;
-};
-
-struct FormatType {
-  char FileDirName[MAX_FILENAME_LENGTH];
-  struct RawEntryType *rawEntries;
-  int n_raw;
-  struct LincomEntryType *lincomEntries;
-  int n_lincom;
-  struct LinterpEntryType *linterpEntries;
-  int n_linterp;
-  struct MplexEntryType *mplexEntries;
-  int n_mplex;
-  struct BitEntryType *bitEntries;
-  int n_bit;
-};
-
 static struct {
   int n;
   struct FormatType *F;
@@ -287,7 +239,7 @@ static int BitCmp(const void *A, const void *B) {
 /*      is cached.                                                         */
 /*                                                                         */
 /***************************************************************************/
-static struct FormatType *GetFormat(char *filedir, int *error_code) {
+struct FormatType *GetFormat(const char *filedir, int *error_code) {
   int i_format;
   FILE *fp;
   char format_file[MAX_FILENAME_LENGTH+6];
@@ -347,7 +299,6 @@ static struct FormatType *GetFormat(char *filedir, int *error_code) {
       FreeF(F);
       return(NULL);
     }
-    
     if (strcmp(in_cols[1], "RAW")==0) {
       F->n_raw++;
       F->rawEntries =
@@ -413,7 +364,6 @@ static struct FormatType *GetFormat(char *filedir, int *error_code) {
     qsort(F->bitEntries, F->n_bit, sizeof(struct BitEntryType),
 	  BitCmp);
   }
-
   return(F);
 }
 
