@@ -36,6 +36,7 @@
 #include <kstdaction.h>
 #include <kaccel.h>
 #include <kstatusbar.h>
+#include <kkeydialog.h>
 
 #include <iostream>
 
@@ -58,7 +59,7 @@
 #include "kstdatamanager_i.h"
 #include "kstlabeldialog_i.h"
 #include "kstiface_impl.h"
-//#include "resource.h"
+#include "vectorsavedialog.h"
 
 #define KST_STATUSBAR_FRAME 0
 #define KST_STATUSBAR_DATA 1
@@ -67,6 +68,8 @@
 KstApp::KstApp(QWidget *parent, const char* name)
 : KMainWindow(parent, name) {
 
+  clearWFlags(WDestructiveClose);
+
   stopping = false;
   config = kapp->config();
   initStatusBar();
@@ -82,6 +85,7 @@ KstApp::KstApp(QWidget *parent, const char* name)
   quickCurvesDialog = new KstQuickCurvesDialogI(this);
   quickPSDDialog = new KstQuickPSDDialogI(this);
   graphFileDialog = new KstGraphFileDialogI(this);
+  vectorSaveDialog = new VectorSaveDialog(this);
   connect(KstVectorDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstCurveDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
   connect(KstEqDialogI::globalInstance(), SIGNAL(modified()), doc, SLOT(wasModified()));
@@ -165,7 +169,7 @@ void KstApp::initActions() {
   KStdAction::saveAs( this, SLOT(slotFileSaveAs()), actionCollection());
   KStdAction::close(  this, SLOT(slotFileClose()), actionCollection());
   KStdAction::quit(   this, SLOT(slotFileClose()), actionCollection());
-
+  KStdAction::keyBindings( this, SLOT( slotConfigureKeys() ), actionCollection() );
   /************/
   filePrint = KStdAction::print(  this, SLOT(slotFilePrint()),
 				  actionCollection());
@@ -338,6 +342,15 @@ void KstApp::initActions() {
                                            "to export the plot as a\n"
                                            "graphics file\n"));
 
+  /************/
+  _vectorSaveAction = new KAction(i18n("Save Vectors to Disk..."),
+                                  0, 0,
+                                  vectorSaveDialog, SLOT(show()),
+                                  actionCollection(),
+                                  "vectorsavedialog_action");
+  _vectorSaveAction->setWhatsThis(i18n("Bring up a dialog box\n"
+                                       "to save vectors to text files.\n"));
+
   /************/
   SamplesDownAction = new KAction(i18n("&Back 1 Screen"),
                                   "kst_back",
@@ -379,6 +392,13 @@ void KstApp::initActions() {
   createGUI();
 }
 
+
+void KstApp::slotConfigureKeys()
+{
+  KKeyDialog::configure( actionCollection(), this );
+}
+
+
 void KstApp::setPaused(bool in_paused) {
   PauseAction->setChecked(in_paused);
 }
@@ -408,10 +428,8 @@ bool KstApp::getTieZoom() {
 }
 
 void KstApp::initStatusBar() {
-  statusBar()->insertFixedItem(i18n("Frame: 00000000"),
-                               KST_STATUSBAR_FRAME);
-  statusBar()->setItemAlignment(KST_STATUSBAR_FRAME,
-                                AlignLeft | AlignVCenter);
+  statusBar()->insertFixedItem(i18n("Frame: 00000000"), KST_STATUSBAR_FRAME);
+  statusBar()->setItemAlignment(KST_STATUSBAR_FRAME, AlignLeft | AlignVCenter);
   statusBar()->changeItem(i18n("Frame: 0"), KST_STATUSBAR_FRAME);
 
   statusBar()->insertFixedItem("(000000000, 000000000)",
@@ -549,7 +567,7 @@ void KstApp::slotFileOpen() {
     QString fileToOpen=
       KFileDialog::getOpenFileName("::<kstfiledir>",
 				   i18n("*.kst|Kst Plot File "
-					"(*.kst)\n*|All files"),
+					"(*.kst)\n*|All Files"),
                                    this, i18n("Open File"));
     if(!fileToOpen.isEmpty()) {
       doc->openDocument(fileToOpen);
@@ -609,12 +627,11 @@ void KstApp::slotFileSaveAs() {
 
 void KstApp::slotFileClose() {
 
-
-  doc->cancelUpdate();
-  stopping = true;
-  kapp->processEvents();
-
   if (doc->saveModified()) {
+    doc->cancelUpdate();
+    stopping = true;
+    kapp->processEvents();
+
     doc->deleteContents();
     close();
   }
@@ -704,7 +721,7 @@ void KstApp::slotTimer() {
 
   // pause during mouse zooming
   KstMouseModeType M = view->MouseInfo->getMode();
-  if ((M==XY_ZOOMBOX) || (M == Y_ZOOMBOX) || (M==X_ZOOMBOX)) {
+  if (M == XY_ZOOMBOX || M == Y_ZOOMBOX || M == X_ZOOMBOX) {
     timer->start(200);
     return;
   }
@@ -774,6 +791,7 @@ void KstApp::updateDialogs() {
   quickCurvesDialog->update();
   quickPSDDialog->update();
   dataManager->update();
+  vectorSaveDialog->init();
   view->update();
 }
 
