@@ -108,7 +108,7 @@ void KstChangeFileDialogI::applyFileChange() {
   if (it == KST::fileList.end()) {
     file = new KstFile(ChangeFileName->currentText());
     if (file->numFrames() < 1) { // No data in file
-      KMessageBox::sorry(0L, i18n("%1: The requested file does not contain data. Operation cancelled.").arg(ChangeFileName->currentText()));
+      KMessageBox::sorry(0L, i18n("%1: The requested file does not contain data. Operation canceled.").arg(ChangeFileName->currentText()));
       delete file;
       return;
     }
