@@ -541,9 +541,9 @@ void KstDoc::purge() {
   bool modified = false;
   // ASSUMPTION: this only gets called from the data manager!
   for (KstDataObjectList::Iterator it = KST::dataObjectList.begin(); it != KST::dataObjectList.end(); ++it) {
-    kdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+    //kdDebug() << "OBJECT: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
     if ((*it)->getUsage() == 0) {
-      kdDebug() << "    -> REMOVED" << endl;
+      //kdDebug() << "    -> REMOVED" << endl;
       KstDataObjectList::Iterator byebye = it;
       --it;
       KST::dataObjectList.remove(byebye);
@@ -553,9 +553,9 @@ void KstDoc::purge() {
 
   KstRVectorList rvl = kstObjectSubList<KstVector,KstRVector>(KST::vectorList);
   for (KstRVectorList::Iterator it = rvl.begin(); it != rvl.end(); ++it) {
-    kdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
+    //kdDebug() << "VECTOR: " << (*it)->tagName() << " USAGE: " << (*it)->getUsage() << endl;
     if ((*it)->getUsage() == 1) {
-      kdDebug() << "    -> REMOVED" << endl;
+      //kdDebug() << "    -> REMOVED" << endl;
       KST::vectorList.remove((*it).data());
       modified = true;
     }
