@@ -32,6 +32,9 @@
 #include "kstequationcurve.h"
 #include "ksteqdialog_i.h"
 
+const QString KstEquationCurve::XVECTOR = "X";
+const QString KstEquationCurve::OUTVECTOR = "O"; // Output (slave) vector
+
 KstEquationCurve::KstEquationCurve(const QString &in_tag,
                                    const QString &equation,
                                    double x0, double x1,
@@ -39,7 +42,7 @@ KstEquationCurve::KstEquationCurve(const QString &in_tag,
 : KstBaseCurve() {
   _staticX = true;
   DoInterp = false;
-  _inputVectors.prepend(KstVector::generateVector(x0, x1, nx, QString::null));
+  _inputVectors.insert(XVECTOR, KstVector::generateVector(x0, x1, nx, QString::null));
   commonConstructor(in_tag, equation, in_color);
   update();
 }
@@ -52,7 +55,7 @@ KstEquationCurve::KstEquationCurve(const QString &in_tag,
 : KstBaseCurve() {
   _staticX = false;
   DoInterp = do_interp; //false;
-  _inputVectors.prepend(xvector);
+  _inputVectors.insert(XVECTOR, xvector);
 
   commonConstructor(in_tag, equation, in_color);
   update();
@@ -91,7 +94,7 @@ KstEquationCurve::KstEquationCurve(QDomElement &e)
       } else if (e.tagName() == "xvtag") {
         xvtag = e.text();
       } else if (e.tagName() == "xvector") {
-        _inputVectorLoadQueue.prepend(e.text());
+        _inputVectorLoadQueue.append(qMakePair(XVECTOR, e.text()));
       } else if (e.tagName() == "color") {
         in_color.setNamedColor(e.text());
       } else if (e.tagName() == "hasLines") {
@@ -116,29 +119,24 @@ KstEquationCurve::KstEquationCurve(QDomElement &e)
     }
     _staticX = true;
     DoInterp = false;
-    _inputVectors.prepend(KstVector::generateVector(x0, x1, ns, xvtag));
+    _inputVectors.insert(XVECTOR, KstVector::generateVector(x0, x1, ns, xvtag));
   }
   commonConstructor(in_tag, equation, in_color);
 }
 
 KstEquationCurve::~KstEquationCurve() {
   if (_staticX) {
-    KST::vectorList.remove(_inputVectors[0]);
+    KST::vectorList.remove(_inputVectors[XVECTOR]);
   }
 }
 
 
 void KstEquationCurve::loadInputs() {
   if (!_staticX) {
-    KstVectorPtr in_V = *KST::vectorList.findTag(_inputVectorLoadQueue[0]);
-    if (!in_V.data()) {
-      kdWarning() << "Unable to find required vector: "
-                  << _inputVectorLoadQueue[0] << endl;
-    }
-    _inputVectors.prepend(in_V);
+    KstDataObject::loadInputs();
+  } else {
+    update();
   }
-
-  update();
 }
 
 void KstEquationCurve::commonConstructor(const QString &in_tag,
@@ -156,7 +154,7 @@ void KstEquationCurve::commonConstructor(const QString &in_tag,
   _tag = in_tag;
 
   KstVectorPtr yv = new KstVector(_tag + "-sv", NS);
-  _outputVectors.append(yv);
+  _outputVectors.insert(OUTVECTOR, yv);
   yv->zero();
 
   IsValid = false;
@@ -173,7 +171,7 @@ KstObject::UpdateType KstEquationCurve::update(int update_counter) {
   if (KstObject::checkUpdateCounter(update_counter))
     return NO_CHANGE;
 
-  if (update_counter<=0){
+  if (update_counter <= 0){
     force = true;
   } else {
     unsigned int i;
@@ -182,10 +180,10 @@ KstObject::UpdateType KstEquationCurve::update(int update_counter) {
     }
   }
 
-  MaxX = _inputVectors[0]->max();
-  MinX = _inputVectors[0]->min();
-  MeanX= _inputVectors[0]->mean();
-  MinPosX = _inputVectors[0]->minPos();
+  MaxX = _inputVectors[XVECTOR]->max();
+  MinX = _inputVectors[XVECTOR]->min();
+  MeanX= _inputVectors[XVECTOR]->mean();
+  MinPosX = _inputVectors[XVECTOR]->minPos();
 
   preProcess();
 
@@ -193,19 +191,19 @@ KstObject::UpdateType KstEquationCurve::update(int update_counter) {
 
   IsValid = FillY(force) && IsValid;
 
-  _outputVectors[0]->update(update_counter);
+  _outputVectors[OUTVECTOR]->update(update_counter);
 
-  MaxY = _outputVectors[0]->max();
-  MinY = _outputVectors[0]->min();
-  MeanY = _outputVectors[0]->mean();
-  MinPosY = _outputVectors[0]->minPos();
+  MaxY = _outputVectors[OUTVECTOR]->max();
+  MinY = _outputVectors[OUTVECTOR]->min();
+  MeanY = _outputVectors[OUTVECTOR]->mean();
+  MinPosY = _outputVectors[OUTVECTOR]->minPos();
 
   return UPDATE;
 }
 
 void KstEquationCurve::getPoint(int i, double &x, double &y) {
-  x = _inputVectors[0]->interpolate(i, NS);
-  y = _outputVectors[0]->value()[i];
+  x = _inputVectors[XVECTOR]->interpolate(i, NS);
+  y = _outputVectors[OUTVECTOR]->value()[i];
 }
 
 void KstEquationCurve::save(QTextStream &ts) {
@@ -213,12 +211,12 @@ void KstEquationCurve::save(QTextStream &ts) {
   ts << "  <tag>" << _tag << "</tag>" << endl;
   ts << "  <equation>" << Equation << "</equation>" << endl;
   if (_staticX) {
-    ts << "  <x0>" << QString::number(_inputVectors[0]->min()) << "</x0>" << endl;
-    ts << "  <x1>" << QString::number(_inputVectors[0]->max()) << "</x1>" << endl;
-    ts << "  <ns>" << QString::number(_inputVectors[0]->sampleCount()) << "</ns>" << endl;
-    ts << "  <xvtag>" << _inputVectors[0]->tagName() << "</xvtag>" << endl;
+    ts << "  <x0>" << QString::number(_inputVectors[XVECTOR]->min()) << "</x0>" << endl;
+    ts << "  <x1>" << QString::number(_inputVectors[XVECTOR]->max()) << "</x1>" << endl;
+    ts << "  <ns>" << QString::number(_inputVectors[XVECTOR]->sampleCount()) << "</ns>" << endl;
+    ts << "  <xvtag>" << _inputVectors[XVECTOR]->tagName() << "</xvtag>" << endl;
   } else {
-    ts << "  <xvector>" << _inputVectors[0]->tagName() << "</xvector>" << endl;
+    ts << "  <xvector>" << _inputVectors[XVECTOR]->tagName() << "</xvector>" << endl;
     if (DoInterp) {
       ts << "  <interpolate/>" << endl;
     }
@@ -238,10 +236,10 @@ void KstEquationCurve::setEquation(const QString &in_fn) {
 
 void KstEquationCurve::setExistingXVector(KstVectorPtr in_xv, bool do_interp) {
   if (_staticX) {
-    KST::vectorList.remove(_inputVectors[0]);
+    KST::vectorList.remove(_inputVectors[XVECTOR]);
   }
 
-  _inputVectors[0] = in_xv;
+  _inputVectors[XVECTOR] = in_xv;
 
   NS = 2; // reset the updating
   _staticX = false;
@@ -250,10 +248,10 @@ void KstEquationCurve::setExistingXVector(KstVectorPtr in_xv, bool do_interp) {
 
 void KstEquationCurve::setStaticXVector(double xmin, double xmax, int n) {
   if (_staticX) {
-    KST::vectorList.remove(_inputVectors[0]);
+    KST::vectorList.remove(_inputVectors[XVECTOR]);
   }
   _staticX = true;
-  _inputVectors[0] = KstVector::generateVector(xmin, xmax, n, QString::null);
+  _inputVectors[XVECTOR] = KstVector::generateVector(xmin, xmax, n, QString::null);
 }
 
 
@@ -262,7 +260,7 @@ QString KstEquationCurve::getYLabel() const {
 }
 
 QString KstEquationCurve::getXLabel() const {
-  return _inputVectors[0]->label();
+  return _inputVectors[XVECTOR]->label();
 }
 
 KstCurveType KstEquationCurve::type() const {
@@ -271,9 +269,9 @@ KstCurveType KstEquationCurve::type() const {
 
 void KstEquationCurve::setTagName(const QString &in_tag) {
   _tag = in_tag;
-  _outputVectors[0]->setTagName(in_tag+"-sv");
+  _outputVectors[OUTVECTOR]->setTagName(in_tag+"-sv");
   if (_staticX) {
-    _inputVectors[0]->setTagName(in_tag);
+    _inputVectors[XVECTOR]->setTagName(in_tag);
   }
 }
 
@@ -504,14 +502,14 @@ bool KstEquationCurve::FillY(bool force) {
 
   // determine value of Interp
   if (DoInterp) {
-    ns = _inputVectors[0]->sampleCount();
+    ns = _inputVectors[XVECTOR]->sampleCount();
     for (i_v=0; i_v < VectorsUsed.count(); i_v++) {
       if (VectorsUsed[i_v]->sampleCount() > ns) {
         ns = VectorsUsed[i_v]->sampleCount();
       }
     }
-    if (_inputVectors[0]->sampleCount() > 0) {
-      Interp = ns / _inputVectors[0]->sampleCount();
+    if (_inputVectors[XVECTOR]->sampleCount() > 0) {
+      Interp = ns / _inputVectors[XVECTOR]->sampleCount();
     } else { // avoid divide by zero
       Interp = 1;
     }
@@ -519,11 +517,11 @@ bool KstEquationCurve::FillY(bool force) {
     Interp = 1;
   }
 
-  if (NS != _inputVectors[0]->sampleCount() || Interp != 1 ||
-      _inputVectors[0]->numShift() != _inputVectors[0]->numNew() || ScalarsUsed.count()) {
-    NS = _inputVectors[0]->sampleCount()*Interp;
+  if (NS != _inputVectors[XVECTOR]->sampleCount() || Interp != 1 ||
+      _inputVectors[XVECTOR]->numShift() != _inputVectors[XVECTOR]->numNew() || ScalarsUsed.count()) {
+    NS = _inputVectors[XVECTOR]->sampleCount()*Interp;
 
-    KstVectorPtr yv = _outputVectors[0];
+    KstVectorPtr yv = _outputVectors[OUTVECTOR];
     yv->resize(NS);
     yv->zero();
     i0 = 0; // other vectors may have diffent lengths, so start over
@@ -535,8 +533,8 @@ bool KstEquationCurve::FillY(bool force) {
   } else {
     // calculate shift and new samples
     // only do shift optimization if all used vectors are same size and shift
-    v_shift = _inputVectors[0]->numShift()*Interp;
-    v_new = _inputVectors[0]->numNew()*Interp;
+    v_shift = _inputVectors[XVECTOR]->numShift()*Interp;
+    v_new = _inputVectors[XVECTOR]->numNew()*Interp;
 
     for (i_v = 0; i_v < VectorsUsed.count(); i_v++) {
       if (v_shift != VectorsUsed[i_v]->numShift())
@@ -551,7 +549,7 @@ bool KstEquationCurve::FillY(bool force) {
       i0 = 0;
       v_shift = NS;
     } else {
-      KstVectorPtr yv = _outputVectors[0];
+      KstVectorPtr yv = _outputVectors[OUTVECTOR];
       for (i = v_shift; i < NS; i++) {
         yv->value()[i - v_shift] = yv->value()[i];
       }
@@ -559,15 +557,15 @@ bool KstEquationCurve::FillY(bool force) {
     }
   }
 
-  NumShifted = _outputVectors[0]->numShift() + v_shift;
+  NumShifted = _outputVectors[OUTVECTOR]->numShift() + v_shift;
   if (NumShifted > NS)
     NumShifted = NS;
 
-  NumNew = NS - i0 + _outputVectors[0]->numNew();
+  NumNew = NS - i0 + _outputVectors[OUTVECTOR]->numNew();
   if (NumNew > NS)
     NumNew = NS;
 
-  _outputVectors[0].data()->SetNewAndShift(NumNew, NumShifted);
+  _outputVectors[OUTVECTOR].data()->SetNewAndShift(NumNew, NumShifted);
 
   if (i0 == 0) {
     for (i = 0; i < MAX_DIV_REG; i++)
@@ -575,8 +573,8 @@ bool KstEquationCurve::FillY(bool force) {
     DivRegPtr = 0;
   }
 
-  double *rawv = _outputVectors[0]->value();
-  KstVectorPtr iv = _inputVectors[0];
+  double *rawv = _outputVectors[OUTVECTOR]->value();
+  KstVectorPtr iv = _inputVectors[XVECTOR];
 
   for(i = i0; i < NS; ++i) {
     //X = (float)i*x_step + MinX;
