@@ -28,9 +28,10 @@
 
 #include <qfile.h>
 #include <ktempfile.h>
-//#include <klocale.h>
+#include <kdebug.h>
 
 #include "creaddata_cpp.h"
+#include "getdata_struct.h"
 #include "getdata_cpp.h"
 
 #include "kstfile.h"
@@ -62,7 +63,7 @@ KstFile::KstFile(QDomElement &e)
   }
 
   /* Call the common constructor */
-  commonConstructor(filename_in, UNKNOWN);
+ commonConstructor(filename_in, UNKNOWN);
 }
 
 void KstFile::commonConstructor(const QString &filename_in,
@@ -144,16 +145,14 @@ int KstFile::readField(double *V, const QString &field, int s,
   case UNKNOWN:
     return 0;
   default:
-    fprintf(stderr,"kst internal error: unknown type in ReadField\n");
+    kdWarning() << "kst internal error: unknown type in ReadField" << endl;
     return 0;
   }
 }
 
 /** Returns the size of the file as of last update */
 int KstFile::numFrames() const {
-  if (Type == EMPTY)
-    return 0;
-  else return NumFrames;
+  return Type == EMPTY ? 0 : NumFrames;
 }
 
 /** Returns true if the field is valid, or false if it is not */
@@ -173,8 +172,8 @@ bool KstFile::isValidField(const QString &field) const {
   case UNKNOWN:
     return false;
   default:
-    fprintf(stderr,"kst internal error: unknown type in isvalidfield\n");
-   return 0;
+    kdWarning() << "kst internal error: unknown type in isValidField" << endl;
+    return false;
   }
 }
 
@@ -195,8 +194,8 @@ int KstFile::samplesPerFrame(const QString &field){
   case UNKNOWN:
     return 0;
   default:
-    fprintf(stderr,"kst internal error: unknown type in sampsperframe\n");
-   return 0;
+    kdWarning() << "kst internal error: unknown type in samplesPerFrame" << endl;
+    return 0;
   }
 }
 
@@ -210,7 +209,8 @@ KstObject::UpdateType KstFile::update(int) {
       init();
   }
 
-  if (IsStdin) UpdateStdin();
+  if (IsStdin)
+    UpdateStdin();
 
   if (Type == EMPTY)
     init();
@@ -226,9 +226,9 @@ KstObject::UpdateType KstFile::update(int) {
   case UNKNOWN:
     return NO_CHANGE;
   default:
-    fprintf(stderr,"kst internal error: unknown type in Update\n");
-    exit(0);
+    kdFatal() << "kst internal error: unknown type in update" << endl;
   }
+  return NO_CHANGE;
 }
 
 /** Returns the file name.  It is stored in a separate static variable,
@@ -259,18 +259,24 @@ QString KstFile::fileType() const {
     break;
   }
 
-  fprintf(stderr,"kst internal error: unknown type in FileType\n");
+  kdWarning() << "kst internal error: unknown type in FileType" << endl;
   return "???";
 }
 
 /** Save file description info into stream ts */
 void KstFile::save(QTextStream &ts) {
-
   ts << "  <filename>";
-  if (IsIndirect) ts << IndirectFilename;
-  else ts << Filename;
-  ts << "</filename>\n";
+  if (IsIndirect) {
+    ts << IndirectFilename;
+  } else {
+    ts << Filename;
+  }
+  ts << "</filename>" << endl;
+}
 
+/** return a qstringlist of valid fields for this file type */
+QStringList KstFile::fieldList() const {
+  return (_fieldList);
 }
 
 /************************************************************************/
@@ -289,9 +295,9 @@ int KstFile::determineType(){
   /**********************************************************/
   /* check to see if the file is a dirfile file (directory) */
   NumFrames = GetNFrames(CFilename, &error_code, NULL);
-  if ((NumFrames>0) && (error_code == GD_E_OK)) {
+  if (NumFrames > 0 && error_code == GD_E_OK) {
     Type = DIRFILE;
-    return (DIRFILE);
+    return DIRFILE;
   }
 
   /**********************************************************/
@@ -302,7 +308,7 @@ int KstFile::determineType(){
     ByteLength = the_file.size();
   } else {
     Type = EMPTY;
-    return (EMPTY);
+    return EMPTY;
   }
 
   /******************************************************************/
@@ -312,9 +318,9 @@ int KstFile::determineType(){
 	    1,0, /* num sframes, num samps */
 	    'n',(void*)NULL,
 	    &error_code);
-  if (error_code==E_OK) {
+  if (error_code == E_OK) {
     Type = FRAME;
-    return (FRAME);
+    return FRAME;
   }
 
   /********************************************/
@@ -325,8 +331,8 @@ int KstFile::determineType(){
   the_file.open(IO_ReadOnly);
   nread = the_file.readBlock(readbuf, 1024);
 
-  bool non_ascii=false;
-  for (i=0; i<nread; i++) {
+  bool non_ascii = false;
+  for (i = 0; i < nread; i++) {
     if (!(isprint(readbuf[i]) || readbuf[i]=='\r' || readbuf[i]=='\n' ||
         readbuf[i]=='\t')) {
       non_ascii = true;
@@ -336,10 +342,10 @@ int KstFile::determineType(){
 
   if (!non_ascii) {
     Type = ASCII;
-    return (ASCII);
+    return ASCII;
   } else {
     Type = UNKNOWN;
-    return (UNKNOWN);
+    return UNKNOWN;
   }
 }
 
@@ -353,15 +359,15 @@ int KstFile::readIFile() {
     QString new_filename;
     ifile.readLine(new_filename, 1024);
     if (new_filename.stripWhiteSpace() == Filename) {
-      return (0);
+      return 0;
     } else {
       setFilename(new_filename.stripWhiteSpace());
-      return(1);
+      return 1;
     }
   } else {
     Type = EMPTY;
     setFilename("");
-    return (0);
+    return 0;
   }
 }
 
@@ -369,6 +375,8 @@ int KstFile::readIFile() {
 /** Initializes files */
 void KstFile::init() {
 
+  _fieldList.clear();
+
   if ((Type == UNKNOWN) || (Type == EMPTY)) {
     determineType();
   }
@@ -386,8 +394,7 @@ void KstFile::init() {
   case UNKNOWN:
     return;
   default:
-    fprintf(stderr,"kst internal error: unknown type in inializer\n");
-    exit(0);
+    kdFatal() << "kst internal error: unknown type in inializer" << endl;
   }
 
 }
@@ -617,8 +624,7 @@ bool KstFile::frameInitFile(){
 	    &error_code);
 
   if (error_code!=E_OK) {
-    std::cerr << "Problem reading FFINFO in from Frame Type File\n";
-    exit(0);
+    kdFatal() << "Problem reading FFINFO in from Frame Type File" << endl;
   }
 
   BytePerFrame = buf[0];
@@ -635,18 +641,18 @@ bool KstFile::frameInitFile(){
   RootFileName[252] = '\0';
 
   if (isxdigit(ext[0]) && isxdigit(ext[1])) {
-    RootFileName[len-2]='\0';
+    RootFileName[len-2] = '\0';
     RootExt = strtol(ext, &tmpstr, 16);
-    MaxExt=RootExt;
+    MaxExt = RootExt;
   } else {
     MaxExt = RootExt = -1;
   }
-  return(frameUpdate());
+  return frameUpdate();
 }
 
 /** Update Frame Data file: determine length return 1 if new data */
 bool KstFile::frameUpdate(){
-  char tmpfilename[255];
+  QString tmpfilename;
   struct stat stat_buf;
   int done = 0;
   int dec = 0;
@@ -654,21 +660,21 @@ bool KstFile::frameUpdate(){
   bool isnew;
 
   if (MaxExt<0) { // no hex number extension: only this file
-    if (stat (CFilename, &stat_buf) != 0) { // file is gone
-      newN=0;
+    if (stat(CFilename, &stat_buf) != 0) { // file is gone
+      newN = 0;
     } else {                               // file exists
       newN = stat_buf.st_size/BytePerFrame;
     }
   } else {
     do {
-      sprintf(tmpfilename,"%s%2.2x",RootFileName, MaxExt);
-      if (stat(tmpfilename, &stat_buf)!=0) { // no file with MaxExt
-	if (MaxExt>RootExt) {  // deleted (?) check the next one down
+      tmpfilename.sprintf("%s%2.2x", RootFileName, MaxExt);
+      if (stat(QFile::encodeName(tmpfilename).data(), &stat_buf) != 0) { // no file with MaxExt
+	if (MaxExt > RootExt) {  // deleted (?) check the next one down
 	  MaxExt--;
 	  dec = 1;
 	} else {                      // All files have been deleted
 	  stat_buf.st_size = 0;
-	  done=1;
+	  done = 1;
 	}
       } else {
 	if (stat_buf.st_size == BytePerFrame*FramePerFile) { // Full file
@@ -728,7 +734,7 @@ int KstFile::frameReadField(double *V, const QString &field,
   int error_code;
   int n_read;
 
-  if (n<0) {
+  if (n < 0) {
     n_read = CReadData(CFilename, field.left(8).latin1(),
                        s,0, /* 1st sframe, 1st samp */
                        0,1, /* num sframes, num samps */
@@ -742,7 +748,7 @@ int KstFile::frameReadField(double *V, const QString &field,
                        &error_code);
   }
 
-  return(n_read);
+  return n_read;
 
 }
 
@@ -754,21 +760,36 @@ int KstFile::frameReadField(double *V, const QString &field,
 
 /** Initialization for Dirfile Files */
 bool KstFile::dirfileInitFile(){
+  struct FormatType *F;
+  int error_code=0;
 
-  NumFrames=0;
+  NumFrames = 0;
+  F = GetFormat(CFilename, &error_code);
+  if (error_code == GD_E_OK) {
+    int i;
 
-  return(dirfileUpdate());
+
+    for (i=0; i<F->n_lincom; i++) {
+      _fieldList.append(QString(F->lincomEntries[i].field));
+    }
+    for (i=0; i<F->n_linterp; i++) {
+      _fieldList.append(QString(F->linterpEntries[i].field));
+    }
+    for (i=0; i<F->n_bit; i++) {
+      _fieldList.append(QString(F->bitEntries[i].field));
+    }
+    for (i=0; i<F->n_raw; i++) {
+      _fieldList.append(QString(F->rawEntries[i].field));
+    }
+  }
+  return dirfileUpdate();
 }
 
 /** Update Dirfile Data file: determine length return 1 if new data */
 bool KstFile::dirfileUpdate(){
-  int newNF;
-  bool isnew;
   int error_code;
-
-  newNF = GetNFrames(CFilename, &error_code, NULL);
-
-  isnew = (newNF != NumFrames);
+  int newNF = GetNFrames(CFilename, &error_code, NULL);
+  bool isnew = newNF != NumFrames;
 
   NumFrames = newNF;
 
@@ -804,7 +825,7 @@ int KstFile::dirfileReadField(double *V, const QString &field,
   int error_code;
   int n_read;
 
-  if (n<0) {
+  if (n < 0) {
     n_read = GetData(CFilename, field.left(16).latin1(),
                        s,0, /* 1st sframe, 1st samp */
                        0,1, /* num sframes, num samps */
@@ -823,13 +844,13 @@ int KstFile::dirfileReadField(double *V, const QString &field,
 }
 
 void KstFile::setFilename(QString new_name) {
-  Filename=new_name;
+  Filename = new_name;
   EncodedFilename = QFile::encodeName(Filename);
   CFilename = EncodedFilename.data();
 }
 
 void KstFile::setFilename(const char *new_name) {
-  Filename=new_name;
+  Filename = new_name;
   EncodedFilename = QFile::encodeName(Filename);
   CFilename = EncodedFilename.data();
 }
