@@ -18,9 +18,10 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
-#include <iostream>
 
+#include <kdebug.h>
 #include <qptrlist.h>
+#include <qstylesheet.h>
 #include <qtextstream.h>
 
 
@@ -71,9 +72,8 @@ void KstLabel::setJustification(KstJustifyType in_j) {
 
 /** Return the width of the label */
 int KstLabel::width(QPainter &p) {
-
   draw(p, 0, 0, false); // Do a dummy draw to evaluate the width
-  return((int)Width);
+  return int(Width);
 }
 
 int KstLabel::ascent(QPainter &p) {
@@ -85,7 +85,7 @@ int KstLabel::ascent(QPainter &p) {
   Ascent = p.fontMetrics().boundingRect("1234567890").height();
   p.restore();
 
-  return(Ascent);
+  return Ascent;
 }
 
 int KstLabel::lineSpacing(QPainter &p) {
@@ -97,7 +97,7 @@ int KstLabel::lineSpacing(QPainter &p) {
   LineSpacing = p.fontMetrics().lineSpacing();
   p.restore();
 
-  return(LineSpacing);
+  return LineSpacing;
 }
 
 typedef struct FPType {
@@ -113,8 +113,7 @@ typedef struct GreekCharType {
 };
 
 
-void KstLabel::draw(QPainter &p, int px, int py,
-                    bool doDraw) {
+void KstLabel::draw(QPainter &p, int px, int py, bool doDraw) {
   int w,h;
   int s;
   double x,y;
@@ -125,28 +124,28 @@ void KstLabel::draw(QPainter &p, int px, int py,
   const int N_GREEK = 44;
   QString processedText;
 
-  GreekCharType GC[] = {{"\\Alpha", "A"}, {"\\alpha", "a"},
-                        {"\\Beta", "B"}, {"\\beta", "b"},
-                        {"\\Chi", "C"}, {"\\chi", "c"},
-                        {"\\Delta", "D"}, {"\\delta", "d"},
-                        {"\\Epsilon", "E"}, {"\\epsilon", "e"},
-                        {"\\Phi", "F"}, {"\\phi", "f"},
-                        {"\\Gamma", "G"}, {"\\gamma", "g"},
-                        {"\\Eta", "H"}, {"\\eta", "h"},
-                        {"\\Iota", "I"}, {"\\iota", "i"},
-                        {"\\Kappa", "K"}, {"\\kappa", "k"},
-                        {"\\Lambda", "L"}, {"\\lambda", "l"},
-                        {"\\Mu", "M"}, {"\\mu", "m"},
-                        {"\\Nu", "N"}, {"\\nu", "n"},
-                        {"\\Pi", "P"}, {"\\pi", "p"},
-                        {"\\Theta", "Q"}, {"\\theta", "q"},
-                        {"\\Rho", "R"}, {"\\rho", "r"},
-                        {"\\Sigma", "S"}, {"\\sigma", "s"},
-                        {"\\Tau", "T"}, {"\\tau", "t"},
-                        {"\\Omega", "W"}, {"\\omega", "w"},
-                        {"\\Psi", "Y"}, {"\\psi", "y"},
-                        {"\\Zeta", "Z"}, {"\\zeta", "z"},
-                        {"\\sum", ""}, {"\\int", ""}
+  static const GreekCharType GC[] = {{"\\Alpha", "A"}, {"\\alpha", "a"},
+                                     {"\\Beta", "B"}, {"\\beta", "b"},
+                                     {"\\Chi", "C"}, {"\\chi", "c"},
+                                     {"\\Delta", "D"}, {"\\delta", "d"},
+                                     {"\\Epsilon", "E"}, {"\\epsilon", "e"},
+                                     {"\\Phi", "F"}, {"\\phi", "f"},
+                                     {"\\Gamma", "G"}, {"\\gamma", "g"},
+                                     {"\\Eta", "H"}, {"\\eta", "h"},
+                                     {"\\Iota", "I"}, {"\\iota", "i"},
+                                     {"\\Kappa", "K"}, {"\\kappa", "k"},
+                                     {"\\Lambda", "L"}, {"\\lambda", "l"},
+                                     {"\\Mu", "M"}, {"\\mu", "m"},
+                                     {"\\Nu", "N"}, {"\\nu", "n"},
+                                     {"\\Pi", "P"}, {"\\pi", "p"},
+                                     {"\\Theta", "Q"}, {"\\theta", "q"},
+                                     {"\\Rho", "R"}, {"\\rho", "r"},
+                                     {"\\Sigma", "S"}, {"\\sigma", "s"},
+                                     {"\\Tau", "T"}, {"\\tau", "t"},
+                                     {"\\Omega", "W"}, {"\\omega", "w"},
+                                     {"\\Psi", "Y"}, {"\\psi", "y"},
+                                     {"\\Zeta", "Z"}, {"\\zeta", "z"},
+                                     {"\\sum", ""}, {"\\int", ""}
   };
 
   QFont TextFont(FontName, fontSize(p), QFont::Normal, false);
@@ -157,7 +156,7 @@ void KstLabel::draw(QPainter &p, int px, int py,
 
   p.save();
   p.setFont(TextFont);
-  p.translate(px,py);
+  p.translate(px, py);
 
   p.setClipping(false);
 
@@ -170,7 +169,6 @@ void KstLabel::draw(QPainter &p, int px, int py,
 
   p.rotate(Rotation);
 
-
   fP[0].locked = true;
   fP[0].dy = 0;
   fP[0].size = 0;
@@ -178,39 +176,39 @@ void KstLabel::draw(QPainter &p, int px, int py,
   x = y = 0;
 
   switch (Justify) {
-      case (CxBy):
+      case CxBy:
         x = -w/2;
         break;
-      case (CxTy):
+      case CxTy:
         x = -w/2;
         y = h;
         break;
-      case (CxCy):
+      case CxCy:
         x = -w/2;
         y = h/2;
         break;
-      case (LxBy):
+      case LxBy:
         break;
-      case (LxTy):
+      case LxTy:
         y = h;
         break;
-      case (LxCy):
+      case LxCy:
         y = h/2;
         break;
-      case (RxBy):
+      case RxBy:
         x = -w;
         break;
-      case (RxTy):
+      case RxTy:
         x = -w;
         y = h;
         break;
-      case (RxCy):
+      case RxCy:
         x = -w;
         y = h/2;
         break;
       default:
-        std::cerr << "Warning: undefined KstJustify in kstlabel::draw: "
-             << Justify << "\n";
+        kdWarning() << "undefined KstJustify in kstlabel::draw: "
+             << Justify << endl;
         break;
   }
 
@@ -327,15 +325,14 @@ void KstLabel::draw(QPainter &p, int px, int py,
 }
 
 void KstLabel::save(QTextStream &ts) {
-  ts << "    <text>" << Text << "</text>\n";
-  ts << "    <justify>" << Justify << "</justify>\n";
-  ts << "    <rotation>" << Rotation << "</rotation>\n";
-  ts << "    <xpos>" << _x << "</xpos>\n";
-  ts << "    <ypos>" << _y << "</ypos>\n";
-  ts << "    <fontfamily>" << FontName << "</fontfamily>\n";
-  ts << "    <symbolfontfamily>" << SymbolFontName << "</symbolfontfamily>\n";
-  ts << "    <size>" << Size << "</size>\n";
-
+  ts << "    <text>" << QStyleSheet::escape(Text) << "</text>" << endl;
+  ts << "    <justify>" << Justify << "</justify>" << endl;
+  ts << "    <rotation>" << Rotation << "</rotation>" << endl;
+  ts << "    <xpos>" << _x << "</xpos>" << endl;
+  ts << "    <ypos>" << _y << "</ypos>" << endl;
+  ts << "    <fontfamily>" << FontName << "</fontfamily>" << endl;
+  ts << "    <symbolfontfamily>" << SymbolFontName << "</symbolfontfamily>" << endl;
+  ts << "    <size>" << Size << "</size>" << endl;
 }
 
 const QString& KstLabel::text() const {
@@ -343,9 +340,8 @@ const QString& KstLabel::text() const {
 }
 
 void KstLabel::read(QDomElement &e) {
-
   Rotation = 0.0;
-  Text = "";
+  Text = QString::null;
   Justify = CxBy;
   _x = _y = 0.0;
   SymbolFontName = "Symbol";
@@ -372,7 +368,7 @@ void KstLabel::read(QDomElement &e) {
       } else if (e.tagName() == "symbolfontfamily") {
         SymbolFontName = e.text();
       } else if (e.tagName() == "fontsize") {
-        Size = e.text().toInt()-12;
+        Size = e.text().toInt() - 12;
       } else if (e.tagName() == "size") {
         Size = e.text().toInt();
       }
@@ -398,7 +394,8 @@ int KstLabel::fontSize(QPainter &p) {
   int x_pix, y_pix;
   double x_s, y_s, s;
 
-  if (IsSample) return (Size+12);
+  if (IsSample)
+    return Size+12;
 
   x_s = y_s = Size + 12.0;
 
@@ -418,15 +415,49 @@ int KstLabel::fontSize(QPainter &p) {
 
   //if (s<12.0) s = 0.5*s + 6;
 
-  if (s<6.0) s = 6.0;
+  if (s < 6.0)
+    s = 6.0;
 
-  return ((int)s);
+  return int(s);
 }
 
 QString KstLabel::fontName() {
-  return (FontName);
+  return FontName;
 }
 
 void KstLabel::setDoScalarReplacement(bool in_do) {
   doScalarReplacement = in_do;
 }
+
+
+// Use for dragging labels
+KstLabelDrag::KstLabelDrag(QWidget *dragSource, int p, int n, const QPoint& hotSpot, const QPixmap &pm)
+: QDragObject(dragSource), _plot(p), _label(n), _hs(hotSpot) {
+  setPixmap(pm, hotSpot);
+}
+
+
+KstLabelDrag::~KstLabelDrag() {
+}
+
+
+const char *KstLabelDrag::format(int i) const {
+  if (i == 0) {
+    return "application/x-kst-label-number";
+  } else {
+    return 0L;
+  }
+}
+
+
+QByteArray KstLabelDrag::encodedData(const char *mime) const {
+  QByteArray a;
+
+  if (QCString(mime) == "application/x-kst-label-number") {
+    QDataStream ds(a, IO_WriteOnly);
+    ds << _plot << _label << _hs;
+  }
+  return a;
+}
+
+// vim: ts=2 sw=2 et
