@@ -375,12 +375,12 @@ void KstPlot::getScale(double &xmin, double &ymin,
 void KstPlot::getLScale(double &x_min, double &y_min,
                        double &x_max, double &y_max) const {
   if (XLog) {
-    if (XMin>0) {
+    if (XMin > 0) {
       x_min = log10(XMin);
     } else {
       x_min = -350;
     }
-    if (XMax>0) {
+    if (XMax > 0) {
       x_max = log10(XMax);
     } else {
       x_max = -340;
@@ -391,12 +391,12 @@ void KstPlot::getLScale(double &x_min, double &y_min,
   }
 
   if (YLog) {
-    if (YMin>0) {
+    if (YMin > 0) {
       y_min = log10(YMin);
     } else {
       y_min = -350;
     }
-    if (YMax>0) {
+    if (YMax > 0) {
       y_max = log10(YMax);
     } else {
       y_max = -340;
@@ -409,11 +409,11 @@ void KstPlot::getLScale(double &x_min, double &y_min,
 
 
 KstScaleModeType KstPlot::getXScaleMode() const {
-  return (XScaleMode);
+  return XScaleMode;
 }
 
 KstScaleModeType KstPlot::getYScaleMode() const {
-  return (YScaleMode);
+  return YScaleMode;
 }
 
 void KstPlot::setXScaleMode(KstScaleModeType scalemode_in) {
@@ -428,6 +428,10 @@ void KstPlot::addCurve(KstBaseCurve *incurve) {
   Curves.append(incurve);
 }
 
+void KstPlot::removeCurve(KstBaseCurve *incurve) {
+  Curves.remove(incurve);
+}
+
 void KstPlot::updateScale() {
   double mid, delta;
 
@@ -730,14 +734,16 @@ void KstPlot::setBorders(double &xleft_bdr_px, double &xright_bdr_px,
   ytop_bdr_px = 1.3*TopLabel->lineSpacing(p);
   ybot_bdr_px = 1.3*XLabel->lineSpacing(p) + TickLabel->ascent(p);
 
-  for (i=0; i*YTick + Yorg < YMax; i++);
+  for (i = 0; i*YTick + Yorg < YMax; i++)
+     /* do nothing */;
   i--;
   genAxisTikLabel(TmpStr,i*YTick+Yorg, YLog);
   TickLabel->setJustification(RxCy);
   TickLabel->setText(TmpStr);
   xleft_bdr_px = TickLabel->width(p);
   /* now check min label */
-  for (i=-1; i*YTick + Yorg > YMin; i--);
+  for (i = -1; i*YTick + Yorg > YMin; i--)
+     /* do nothing */;
   i++;
   genAxisTikLabel(TmpStr,i*YTick+Yorg, YLog);
   TickLabel->setText(TmpStr);
@@ -780,7 +786,7 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
 
   char TmpStr[120];
 
-  int i, j, ns;
+  int i, j;
   bool overlap, nopoint;
 
   updateScale();
@@ -994,8 +1000,7 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
   }
 
   /*** plot the arbitrary labels **/
-  KstLabel *label;
-  for (label = labelList.first(); label; label = labelList.next()) {
+  for (KstLabel *label = labelList.first(); label; label = labelList.next()) {
     label->draw(p, d2i(xleft_bdr_px + RelPlotRegion.width() * label->x()),
                 d2i(ytop_bdr_px + RelPlotRegion.height() * label->y()));
   }
@@ -1011,16 +1016,60 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
   b_X = Lx - m_X * x_min;
   b_Y = Hy - m_Y * y_min;
 
-  for (unsigned i_curve = 0; i_curve < Curves.count(); i_curve++) {
+  for (int i_curve = 0; i_curve < (int)Curves.count(); i_curve++) {
+
     c = Curves[i_curve];
     overlap = false;
     if (c->sampleCount() > 0) {
+
+      int i0, iN;
+      if (c->xIsRising()) {
+        i0 = 0;
+        c->getPoint(0, rX, rY);
+        if (x_min>rX) {
+          int i_top = c->sampleCount()-1;
+          int i_bot = 0;
+          while (i_bot+1<i_top) {
+            i0 = (i_top+i_bot)/2;
+            c->getPoint(i0, rX, rY);
+            if (x_min<rX) {
+              i_top = i0;
+            } else {
+              i_bot = i0;
+            }
+          }
+          i0 = i_bot;
+        }
+
+        // find Maximum
+        iN = c->sampleCount()-1;
+        c->getPoint(iN, rX, rY);
+        if (x_max<rX) {
+          int i_top = c->sampleCount()-1;
+          int i_bot = 0;
+          while (i_bot+1<i_top) {
+            iN = (i_top+i_bot)/2;
+            c->getPoint(iN, rX, rY);
+            if (x_max<rX) {
+              i_top = iN;
+            } else {
+              i_bot = iN;
+            }
+          }
+          iN = i_top+1;
+        }
+      } else {
+        i0 = 0;
+        iN = c->sampleCount()-1;
+      }
+
       p.setPen(QPen(c->getColor(), penWidth));
 
       if (c->hasLines()) {
+
 	/* Read i_pt = 0 */
         nopoint = false;
-	c->getPoint(0, rX, rY);
+	c->getPoint(i0, rX, rY);
         if (isnan(rX))
           nopoint = true;
         if (isnan(rY))
@@ -1047,8 +1096,7 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
         last_x1i = (int)X1;
 	last_y1=Y1;
 
-        ns = c->sampleCount();
-	for (i_pt = 1; i_pt < ns; i_pt++) {
+	for (i_pt = i0+1; i_pt < iN; i_pt++) {
 
 	  X2 = last_x1;
           X2i = last_x1i;
@@ -1057,7 +1105,7 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
 	  /* read next i_pt */
           nopoint = false;
           c->getPoint(i_pt, rX, rY);
-          while ((isnan(rX) || isnan(rY)) && i_pt < ns) {
+          while ((isnan(rX) || isnan(rY)) && i_pt < iN) {
             nopoint = true;
             i_pt++;
             c->getPoint(i_pt, rX, rY);
@@ -1157,8 +1205,8 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
       } // end if c->hasLines()
 
       if (c->hasPoints()) {
-        ns = c->sampleCount();
-	for (i_pt = 0; i_pt<ns; i_pt++) {
+
+	for (i_pt = i0; i_pt<iN; i_pt++) {
 	  c->getPoint(i_pt, rX, rY);
           if (XLog) {
             if (rX>0) rX = log10(rX);
@@ -1179,8 +1227,8 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
       if (c->hasXError()) {
 	bool do_low_flag = true;
 	bool do_high_flag = true;
-        ns = c->sampleCount();
-	for (i_pt = 0; i_pt<ns; i_pt++) {
+
+	for (i_pt = i0; i_pt<iN; i_pt++) {
 	  do_low_flag = true;
 	  do_high_flag = true;
 	  c->getEXPoint(i_pt, rX, rY, rEX);
@@ -1215,8 +1263,7 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
 	bool do_low_flag = true;
 	bool do_high_flag = true;
 
-        ns = c->sampleCount();
-	for (i_pt = 0; i_pt<ns; i_pt++) {
+	for (i_pt = i0; i_pt<iN; i_pt++) {
 	  do_low_flag = true;
 	  do_high_flag = true;
 	  c->getEYPoint(i_pt, rX, rY, rEY);
@@ -1224,16 +1271,16 @@ void KstPlot::paint(QPainter &p, double in_xleft_bdr_px) {
 	  X1 = m_X*rX + b_X;
 	  Y1 = m_Y*rY + b_Y - EY;
 	  Y2 = m_Y*rY + b_Y + EY;
-	  if ((Y1<Ly) && (Y2>Ly)) {
+	  if (Y1 < Ly && Y2 > Ly) {
 	    Y1 = Ly;
 	    do_low_flag = false;
 	  }
-	  if ((Y1<Hy) && (Y2>Hy)) {
+	  if (Y1 < Hy && Y2 > Hy) {
 	    Y2 = Hy;
 	    do_high_flag = false;
 	  }
 
-	  if ((X1>=Lx) && (X1<=Hx) && (Y1>=Ly) && (Y2<=Hy)) {
+	  if (X1 >= Lx && X1 <= Hx && Y1 >= Ly && Y2 <= Hy) {
 	    p.drawLine((int)X1, (int) Y1, (int) X1, (int) Y2);
 	    if (do_low_flag) {
 	      p.drawLine((int)X1+c->Point.getDim(&p), (int)Y1,
@@ -1298,48 +1345,47 @@ void KstPlot::setpixrect(int x0, int y0) {
 
 
 void KstPlot::save(QTextStream &ts) {
-  int i;
-  ts << "  <tag>" << tagName() << "</tag>\n";
-  ts << "  <width>" << width() << "</width>\n";
-  ts << "  <height>" << height() << "</height>\n";
-  ts << "  <pos_x>" << topLeftX() << "</pos_x>\n";
-  ts << "  <pos_y>" << topLeftY() << "</pos_y>\n";
+  ts << "  <tag>" << tagName() << "</tag>" << endl;
+  ts << "  <width>" << width() << "</width>" << endl;
+  ts << "  <height>" << height() << "</height>" << endl;
+  ts << "  <pos_x>" << topLeftX() << "</pos_x>" << endl;
+  ts << "  <pos_y>" << topLeftY() << "</pos_y>" << endl;
 
-  ts << "  <xscalemode>" << XScaleMode << "</xscalemode>\n";
-  ts << "  <yscalemode>" << YScaleMode << "</yscalemode>\n";
+  ts << "  <xscalemode>" << XScaleMode << "</xscalemode>" << endl;
+  ts << "  <yscalemode>" << YScaleMode << "</yscalemode>" << endl;
 
-  ts << "  <xmin>" << XMin << "</xmin>\n";
-  ts << "  <xmax>" << XMax << "</xmax>\n";
-  ts << "  <ymin>" << YMin << "</ymin>\n";
-  ts << "  <ymax>" << YMax << "</ymax>\n";
+  ts << "  <xmin>" << XMin << "</xmin>" << endl;
+  ts << "  <xmax>" << XMax << "</xmax>" << endl;
+  ts << "  <ymin>" << YMin << "</ymin>" << endl;
+  ts << "  <ymax>" << YMax << "</ymax>" << endl;
 
-  ts << "  <toplabel>\n";
+  ts << "  <toplabel>" << endl;
   TopLabel->save(ts);
-  ts << "  </toplabel>\n";
+  ts << "  </toplabel>" << endl;
 
-  ts << "  <xlabel>\n";
+  ts << "  <xlabel>" << endl;
   XLabel->save(ts);
-  ts << "  </xlabel>\n";
+  ts << "  </xlabel>" << endl;
 
-  ts << "  <ylabel>\n";
+  ts << "  <ylabel>" << endl;
   YLabel->save(ts);
-  ts << "  </ylabel>\n";
+  ts << "  </ylabel>" << endl;
 
-  ts << "  <ticklabel>\n";
+  ts << "  <ticklabel>" << endl;
   TickLabel->save(ts);
-  ts << "  </ticklabel>\n";
+  ts << "  </ticklabel>" << endl;
 
-  if (isXLog()) ts << "  <xlog/>\n";
-  if (isYLog()) ts << "  <ylog/>\n";
+  if (isXLog()) ts << "  <xlog/>" << endl;
+  if (isYLog()) ts << "  <ylog/>" << endl;
 
-  for (i=0; i<(int)labelList.count(); i++) {
-    ts << "  <label>\n";
+  for (unsigned i = 0; i < labelList.count(); i++) {
+    ts << "  <label>" << endl;
     labelList.at(i)->save(ts);
-    ts << "  </label>\n";
+    ts << "  </label>" << endl;
   }
 
-  for (i=0; i<(int)Curves.count(); i++) {
-    ts<<"  <curvetag>" << Curves[i]->tagName() << "</curvetag>\n";
+  for (unsigned i = 0; i < Curves.count(); i++) {
+    ts << "  <curvetag>" << Curves[i]->tagName() << "</curvetag>" << endl;
   }
 
 }
