@@ -38,6 +38,7 @@
 
 #include "kstdatacollection.h"
 #include "kstplotdialog_i.h"
+
 /*
  *  Constructs a KstPlotDialogI which is a child of 'parent', with the
  *  name 'name' and widget flags set to 'f'
@@ -46,21 +47,21 @@
  *  TRUE to construct a modal dialog.
  */
 KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
-                               const char* name, bool modal,
-                               WFlags fl)
-  : KstPlotDialog(parent, name, modal, fl ) {
+                               const char* name, bool modal, WFlags fl)
+: KstPlotDialog(parent, name, modal, fl ) {
 
     doc = in_doc;
     setScalarDestTopLabel();
 
-    connect(Select, SIGNAL(activated(int)),
-            this, SLOT(update(int)));
-    connect(New, SIGNAL(clicked()),
-            this, SLOT(new_I()));
-    connect(Edit, SIGNAL(clicked()),
-            this, SLOT(edit_I()));
-    connect(Delete, SIGNAL(clicked()),
-            this, SLOT(delete_I()));
+    connect(Select, SIGNAL(activated(int)), this, SLOT(update(int)));
+    connect(New, SIGNAL(clicked()), this, SLOT(new_I()));
+    connect(Edit, SIGNAL(clicked()), this, SLOT(edit_I()));
+    connect(Delete, SIGNAL(clicked()), this, SLOT(delete_I()));
+
+    connect(DisplayedCurveList, SIGNAL(selectionChanged(QListBoxItem*)),
+            this, SLOT(updateButtons()));
+    connect(AvailableCurveList, SIGNAL(selectionChanged(QListBoxItem*)),
+            this, SLOT(updateButtons()));
 
     /* adding/removing curves */
     connect(CurveUnplot, SIGNAL(clicked()),
@@ -113,8 +114,7 @@ KstPlotDialogI::KstPlotDialogI(KstDoc *in_doc, QWidget* parent,
     connect(TopLabelText, SIGNAL(textChanged(const QString &)),
             this, SLOT(setScalarDestTopLabel()));
 
-    SampleLabel = new KstLabel("\0", CxCy,
-                               0.0, 0.0, 0.0, true);
+    SampleLabel = new KstLabel("\0", CxCy, 0.0, 0.0, 0.0, true);
 }
 
 KstPlotDialogI::~KstPlotDialogI() {
@@ -153,13 +153,13 @@ void KstPlotDialogI::update(int new_index) {
   }
 
   Select->clear();
-  for (i_plot=0; i_plot<(int)KST::plotList.count(); i_plot++) {
+  for (i_plot = 0; i_plot < (int)KST::plotList.count(); i_plot++) {
     Select->insertItem(KST::plotList.at(i_plot)->tagName());
   }
 
-  if ((index>=0) && (index<Select->count())) {
+  if (index >= 0 && index < Select->count()) {
     Select->setCurrentItem(index);
-  } else if (Select->count()>0) {
+  } else if (Select->count() > 0) {
     Select->setCurrentItem(Select->count()-1);
   }
 
@@ -167,7 +167,7 @@ void KstPlotDialogI::update(int new_index) {
   DisplayedCurveList->clear();
   AvailableCurveList->clear();
 
-  if (Select->count()>0) {
+  if (Select->count() > 0) {
     i_plot = Select->currentItem();
     PlotPosition->setValue(i_plot);
   } else {
@@ -225,21 +225,21 @@ void KstPlotDialogI::update(int new_index) {
       YAutoUp->setChecked(true);
     }
 
-    (NumberFontSize)->setValue(plot->TickLabel->size());
+    NumberFontSize->setValue(plot->TickLabel->size());
 
-    (XAxisText)->setText(plot->XLabel->text());
-    (XLabelFontSize)->setValue(plot->XLabel->size());
+    XAxisText->setText(plot->XLabel->text());
+    XLabelFontSize->setValue(plot->XLabel->size());
 
-    (YAxisText)->setText(plot->YLabel->text());
-    (YLabelFontSize)->setValue(plot->YLabel->size());
+    YAxisText->setText(plot->YLabel->text());
+    YLabelFontSize->setValue(plot->YLabel->size());
 
-    (TopLabelText)->setText(plot->TopLabel->text());
-    (TopLabelFontSize)->setValue(plot->TopLabel->size());
+    TopLabelText->setText(plot->TopLabel->text());
+    TopLabelFontSize->setValue(plot->TopLabel->size());
 
-    (FontComboBox)->clear();
+    FontComboBox->clear();
 
-    (FontComboBox)-> setFonts(qfd.families());
-    (FontComboBox)->setCurrentFont(plot->TopLabel->fontName());
+    FontComboBox->setFonts(qfd.families());
+    FontComboBox->setCurrentFont(plot->TopLabel->fontName());
 
     updateSampXLabel();
     updateSampYLabel();
@@ -253,6 +253,8 @@ void KstPlotDialogI::update(int new_index) {
   PlotCols->setValue(KST::plotList.getPlotCols());
 
   UpdateDelay->setValue(doc->delay());
+
+  updateButtons();
 }
 
 void KstPlotDialogI::applyLabels(KstPlot *plot) {
@@ -300,8 +302,7 @@ void KstPlotDialogI::applyLimits(KstPlot *plot) {
     plot->setXScale(0, XACRange->text().toDouble());
   } else if (XFixed->isChecked()) {
     plot->setXScaleMode(FIXED);
-    plot->setXScale(XFixedMin->text().toDouble(),
-                    XFixedMax->text().toDouble());
+    plot->setXScale(XFixedMin->text().toDouble(), XFixedMax->text().toDouble());
   } else if (XAutoUp->isChecked()) {
     plot->setXScaleMode(AUTOUP);
   } else if (XAuto->isChecked()) {
@@ -530,10 +531,11 @@ void KstPlotDialogI::updateLabels() {
 
 void KstPlotDialogI::removeDisplayedCurve(){
   int i_curve = DisplayedCurveList->currentItem();
-  if (i_curve>=0) {
+  if (i_curve >= 0) {
     AvailableCurveList->insertItem(DisplayedCurveList->currentText());
     DisplayedCurveList->removeItem(i_curve);
   }
+  updateButtons();
 }
 
 void KstPlotDialogI::addDisplayedCurve(){
@@ -542,6 +544,7 @@ void KstPlotDialogI::addDisplayedCurve(){
     DisplayedCurveList->insertItem(AvailableCurveList->currentText());
     AvailableCurveList->removeItem(i_curve);
   }
+  updateButtons();
 }
 
 
@@ -555,16 +558,18 @@ void KstPlotDialogI::applyAutoLabels() {
 }
 
 void KstPlotDialogI::changePosition(int ni) {
-  KstPlot *plot;
-  int ci;
-
-  if (ni >= Select->count())
+  if (ni >= Select->count()) {
     ni = Select->count() - 1;
+  }
 
-  ci = Select->currentItem();
-  plot = KST::plotList.take(ci);
-  KST::plotList.insert(ni, plot);
+  int ci = Select->currentItem();
+  KST::plotList.insert(ni, KST::plotList.take(ci));
   Select->setCurrentItem(ni);
 }
 
+void KstPlotDialogI::updateButtons() {
+  CurvePlot->setEnabled(AvailableCurveList->selectedItem() != 0L);
+  CurveUnplot->setEnabled(DisplayedCurveList->selectedItem() != 0L);
+}
+
 #include "kstplotdialog_i.moc"
