@@ -99,7 +99,7 @@ void KstPluginDialogI::show_I(const QString &field) {
 void KstPluginDialogI::updatePluginList() {
   PluginCollection *pc = PluginCollection::self();
   QString previous = _pluginList[PluginCombo->currentItem()];
-  int newFocus = 0;
+  int newFocus = -1;
 
   const QMap<QString,Plugin::Data>& _pluginMap = pc->pluginList();
 
@@ -119,8 +119,12 @@ void KstPluginDialogI::updatePluginList() {
     cnt++;
   }
 
-  PluginCombo->setCurrentItem(newFocus);
-  pluginChanged(newFocus);
+  if (newFocus != -1) {
+    PluginCombo->setCurrentItem(newFocus);
+  } else {
+    PluginCombo->setCurrentItem(0);
+    pluginChanged(0);
+  }
 }
 
 
@@ -155,6 +159,10 @@ void KstPluginDialogI::update(int new_index) {
     Select->setCurrentItem(oldSelect < Select->count() ? oldSelect : Select->count()-1);
   }
 
+  if (_pluginList.isEmpty()) {
+    updatePluginList();
+  }
+
   int oldPluginIndex = PluginCombo->currentItem();
 
   KstPluginPtr plugin = pl[Select->currentItem()];
@@ -168,7 +176,6 @@ void KstPluginDialogI::update(int new_index) {
     }
 
     // Update input vector and scalar combos
-    int sptr = 0, vptr = 0;
     const QValueList<Plugin::Data::IOValue>& itable = plugin->plugin()->data()._inputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = itable.begin(); it != itable.end(); ++it) {
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
@@ -178,7 +185,7 @@ void KstPluginDialogI::update(int new_index) {
         if (!updateCombos) {
           vs->update();
         }
-        vs->setSelection(plugin->inputVectors()[vptr++]->tagName());
+        vs->setSelection(plugin->inputVectors()[(*it)._name]->tagName());
       } else {
         QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
         assert(field);
@@ -186,12 +193,11 @@ void KstPluginDialogI::update(int new_index) {
         if (!updateCombos) {
           ss->update();
         }
-        ss->setSelection(plugin->inputScalars()[sptr++]->tagName());
+        ss->setSelection(plugin->inputScalars()[(*it)._name]->tagName());
       }
     }
 
     // Update output vector and scalar lineedits
-    sptr = vptr = 0;
     const QValueList<Plugin::Data::IOValue>& otable = plugin->plugin()->data()._outputs;
     for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
       QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
@@ -199,9 +205,9 @@ void KstPluginDialogI::update(int new_index) {
 
       QLineEdit *li = static_cast<QLineEdit*>(field);
       if ((*it)._type == Plugin::Data::IOValue::TableType) { // vector
-        li->setText(plugin->outputVectors()[vptr++]->tagName());
+        li->setText(plugin->outputVectors()[(*it)._name]->tagName());
       } else { // scalar
-        li->setText(plugin->outputScalars()[sptr++]->tagName());
+        li->setText(plugin->outputScalars()[(*it)._name]->tagName());
       }
     }
   } else { // invalid plugin
@@ -229,13 +235,13 @@ bool KstPluginDialogI::saveInputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
       assert(field);
       VectorSelector *vs = static_cast<VectorSelector*>(field);
       KstVectorPtr v = *KST::vectorList.findTag(vs->selectedVector());
-      plugin->inputVectors().append(v);
+      plugin->inputVectors().insert((*it)._name, v);
     } else {
       QObject *field = _frameWidget->child((*it)._name.latin1(), "ScalarSelector");
       assert(field);
       ScalarSelector *ss = static_cast<ScalarSelector*>(field);
       KstScalarPtr s = *KST::scalarList.findTag(ss->selectedScalar());
-      plugin->inputScalars().append(s);
+      plugin->inputScalars().insert((*it)._name, s);
     }
   }
 return true;
@@ -244,7 +250,6 @@ return true;
 
 bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
   const QValueList<Plugin::Data::IOValue>& otable = p->data()._outputs;
-  unsigned si = 0, vi = 0;
   for (QValueList<Plugin::Data::IOValue>::ConstIterator it = otable.begin(); it != otable.end(); ++it) {
     QObject *field = _frameWidget->child((*it)._name.latin1(), "QLineEdit");
     assert(field);
@@ -258,28 +263,22 @@ bool KstPluginDialogI::saveOutputs(KstPluginPtr plugin, KSharedPtr<Plugin> p) {
     QString nt = li->text();
     if ((*it)._type == Plugin::Data::IOValue::TableType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
-        plugin->outputVectors()[vi]->setTagName(nt);
-      } else {
-        if (plugin->outputVectors()[vi]->tagName() != nt) {
+        plugin->outputVectors()[(*it)._name]->setTagName(nt);
+      } else if (plugin->outputVectors()[(*it)._name]->tagName() != nt) {
           while (KST::vectorTagNameNotUnique(nt, false)) {
             nt += "'";
           }
-          plugin->outputVectors()[vi]->setTagName(nt);
-        }
+          plugin->outputVectors()[(*it)._name]->setTagName(nt);
       }
-      vi++;
     } else if ((*it)._type == Plugin::Data::IOValue::FloatType) {
       if (!KST::vectorTagNameNotUnique(nt, false)) {
-        plugin->outputScalars()[si]->setTagName(nt);
-      } else {
-        if (plugin->outputScalars()[si]->tagName() != nt) {
+        plugin->outputScalars()[(*it)._name]->setTagName(nt);
+      } else if (plugin->outputScalars()[(*it)._name]->tagName() != nt) {
           while (KST::vectorTagNameNotUnique(nt, false)) {
             nt += "'";
           }
-          plugin->outputScalars()[si]->setTagName(nt);
-        }
+          plugin->outputScalars()[(*it)._name]->setTagName(nt);
       } 
-      si++;
     }
   }
 
