@@ -16,7 +16,6 @@
  ***************************************************************************/
 
 /** A class for handling power spectra for kst
-
  *@author C. Barth Netterfield
  */
 
@@ -33,6 +32,10 @@
 
 extern "C" void rdft(int n, int isgn, double *a);
 
+static const QString INVECTOR = "I";
+static const QString SVECTOR = "S";
+static const QString FVECTOR = "F";
+
 KstPSDCurve::KstPSDCurve(const QString &in_tag, KstVectorPtr in_V,
                          double in_freq, int in_len,
                          const QString &in_VUnits, const QString &in_RUnits,
@@ -105,7 +108,7 @@ void KstPSDCurve::commonConstructor(const QString &in_tag, KstVectorPtr in_V,
 
   _typeString = i18n("Power Spectrum");
   NumUsed = 0;
-  _inputVectors[0] = in_V;
+  _inputVectors[INVECTOR] = in_V;
   Color = in_color;
   _tag = in_tag;
   Freq = in_freq;
@@ -127,17 +130,17 @@ void KstPSDCurve::commonConstructor(const QString &in_tag, KstVectorPtr in_V,
   setAppodize(true);
 
   KstVectorPtr iv = new KstVector(_tag+"-freq", 2);
-  _outputVectors.prepend(iv);
+  _outputVectors.insert(FVECTOR, iv);
 
   iv = new KstVector(_tag+"-sv", PSDLen);
-  _outputVectors.prepend(iv);
+  _outputVectors.insert(SVECTOR, iv);
 
   update();
 }
 
 KstPSDCurve::~KstPSDCurve() {
-  KST::vectorList.remove(_outputVectors[0]);
-  KST::vectorList.remove(_outputVectors[1]);
+  KST::vectorList.remove(_outputVectors[SVECTOR]);
+  KST::vectorList.remove(_outputVectors[FVECTOR]);
 
   delete[] w;
   w = 0L;
@@ -147,9 +150,9 @@ KstPSDCurve::~KstPSDCurve() {
 
 void KstPSDCurve::GenW() {
   int i;
-  double sW=0;
+  double sW = 0;
 
-  for (i=0; i<ALen; i++) {
+  for (i = 0; i < ALen; i++) {
     //w[i] = 4;
     w[i] = (1 - cos(2*M_PI*double(i)/double(ALen)));
     sW += w[i] * w[i];
@@ -178,23 +181,23 @@ KstObject::UpdateType KstPSDCurve::update(int update_counter) {
   if (KstObject::checkUpdateCounter(update_counter))
     return NO_CHANGE;
 
-  if (update_counter<=0) {
+  if (update_counter <= 0) {
     force = true;
   } else {
-    _inputVectors[0]->update(update_counter);
+    _inputVectors[INVECTOR]->update(update_counter);
   }
 
-  v_len = _inputVectors[0]->sampleCount();
+  v_len = _inputVectors[INVECTOR]->sampleCount();
 
   n_subsets = v_len/PSDLen+1;
 
-  last_n_new += _inputVectors[0]->numNew();
+  last_n_new += _inputVectors[INVECTOR]->numNew();
 
   if ((last_n_new < PSDLen/16) && (n_subsets - last_n_subsets < 1) && !force) {
     return NO_CHANGE;
   }
 
-  psd = _outputVectors[0]->value();
+  psd = _outputVectors[SVECTOR]->value();
 
   for (i_samp = 0; i_samp < PSDLen; i_samp++) {
     psd[i_samp] = 0;
@@ -211,7 +214,7 @@ KstObject::UpdateType KstPSDCurve::update(int update_counter) {
     for (i_samp = 0; i_samp < copyLen; i_samp++) {
       mean += (
         a[i_samp] =
-        _inputVectors[0]->interpolate(i_samp + i_subset*PSDLen, v_len)
+        _inputVectors[INVECTOR]->interpolate(i_samp + i_subset*PSDLen, v_len)
         );
     }
     if (copyLen>1) mean/=(double)copyLen;
@@ -237,7 +240,7 @@ KstObject::UpdateType KstPSDCurve::update(int update_counter) {
     /* sum each bin into psd[] */
     psd[0]+=a[0];
     psd[PSDLen-1] += a[1];
-    for (i_samp=0; i_samp<PSDLen; i_samp++) {
+    for (i_samp=1; i_samp<PSDLen-1; i_samp++) {
       psd[i_samp]+= cabs(a[i_samp*2], a[i_samp*2+1]);
     }
   }
@@ -281,26 +284,26 @@ KstObject::UpdateType KstPSDCurve::update(int update_counter) {
   MinPosX = 1.0/double(NS) * MaxX;
   MeanX = MaxX/2.0;
 
-  double *f = _outputVectors[1]->value();
+  double *f = _outputVectors[FVECTOR]->value();
   f[0] = 0;
   f[1] = Freq/2.0;
 
-  _outputVectors[0]->update(update_counter);
-  _outputVectors[1]->update(update_counter);
+  _outputVectors[SVECTOR]->update(update_counter);
+  _outputVectors[FVECTOR]->update(update_counter);
 
   return UPDATE;
 }
 
 void KstPSDCurve::getPoint(int i, double &x, double &y) {
-  x = _outputVectors[1]->interpolate(i, NS);
+  x = _outputVectors[FVECTOR]->interpolate(i, NS);
 //xxxdouble(i)/double(NS) * MaxX;
-  y = _outputVectors[0]->interpolate(i, NS);
+  y = _outputVectors[SVECTOR]->interpolate(i, NS);
 }
 
 void KstPSDCurve::save(QTextStream &ts) {
   ts << " <psd>\n";
   ts << "  <tag>" << _tag << "</tag>\n";
-  ts << "  <vectag>" << _inputVectors[0]->tagName() << "</vectag>\n";
+  ts << "  <vectag>" << _inputVectors[INVECTOR]->tagName() << "</vectag>\n";
 
   ts << "  <color>" << Color.name() << "</color>\n";
   ts << "  <hasLines>" << HasLines << "</hasLines>\n";
@@ -316,7 +319,7 @@ void KstPSDCurve::save(QTextStream &ts) {
 
 double KstPSDCurve::getFreq() const {
   return Freq;
-};
+}
 
 void KstPSDCurve::setFreq(double in_freq) {
   if (in_freq > 0.0) {
@@ -338,7 +341,7 @@ void KstPSDCurve::setLen(int in_len) {
   }
 
   PSDLen = int(pow(2,Len-1));
-  _outputVectors[0]->resize(PSDLen);
+  _outputVectors[SVECTOR]->resize(PSDLen);
 
   ALen = PSDLen*2;
   delete[] a;
@@ -353,7 +356,7 @@ void KstPSDCurve::setLen(int in_len) {
 }
 
 QString KstPSDCurve::getYLabel() const {
-  return i18n("PSD of %1 (%2/%3^{1/2})").arg(_inputVectors[0]->label()).arg(VUnits).arg(RUnits);
+  return i18n("PSD of %1 (%2/%3^{1/2})").arg(_inputVectors[INVECTOR]->label()).arg(VUnits).arg(RUnits);
 }
 
 QString KstPSDCurve::getXLabel() const {
@@ -361,11 +364,11 @@ QString KstPSDCurve::getXLabel() const {
 }
 
 QString KstPSDCurve::getVTag() const {
-  return _inputVectors[0]->tagName();
+  return _inputVectors[INVECTOR]->tagName();
 }
 
 void KstPSDCurve::setVector(KstVectorPtr new_v) {
-  _inputVectors[0] = new_v;
+  _inputVectors[INVECTOR] = new_v;
 }
 
 KstCurveType KstPSDCurve::type() const {
