@@ -23,6 +23,11 @@
 #include "kstdatacollection.h"
 #include "kstcurvedialog_i.h"
 
+static const QString XVECTOR = "X";
+static const QString YVECTOR = "Y";
+static const QString EXVECTOR = "EX";
+static const QString EYVECTOR = "EY";
+
 KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y,
 		   KstVectorPtr in_EX, KstVectorPtr in_EY,
                    const QColor &in_color)
@@ -41,17 +46,12 @@ KstVCurve::KstVCurve(const QString &in_tag, KstVectorPtr in_X, KstVectorPtr in_Y
 
 KstVCurve::KstVCurve(QDomElement &e)
 : KstBaseCurve(e) {
-  QString in_tag;
+  QString in_tag, xname, yname, exname, eyname;
   QColor in_color(KstColorSequence::next());
 
   setHasPoints(false);
   setHasLines(true);
 
-  _inputVectorLoadQueue.append(QString::null);
-  _inputVectorLoadQueue.append(QString::null);
-  _inputVectorLoadQueue.append(QString::null);
-  _inputVectorLoadQueue.append(QString::null);
-
   /* parse the DOM tree */
   QDomNode n = e.firstChild();
   while( !n.isNull() ) {
@@ -60,13 +60,13 @@ KstVCurve::KstVCurve(QDomElement &e)
       if (e.tagName() == "tag") {
         in_tag = e.text();
       } else if (e.tagName() == "xvectag") {
-        _inputVectorLoadQueue[0] = e.text();
+        xname = e.text();
       } else if (e.tagName() == "yvectag") {
-        _inputVectorLoadQueue[1] = e.text();
+        yname = e.text();
       } else if (e.tagName() == "exVectag") {
-        _inputVectorLoadQueue[2] = e.text();
+        exname = e.text();
       } else if (e.tagName() == "eyVectag") {
-        _inputVectorLoadQueue[3] = e.text();
+        eyname = e.text();
       } else if (e.tagName() == "color") {
         in_color.setNamedColor(e.text());
       } else if (e.tagName() == "hasLines") {
@@ -80,6 +80,10 @@ KstVCurve::KstVCurve(QDomElement &e)
     n = n.nextSibling();
   }
 
+  _inputVectorLoadQueue.append(qMakePair(XVECTOR, xname));
+  _inputVectorLoadQueue.append(qMakePair(YVECTOR, yname));
+  _inputVectorLoadQueue.append(qMakePair(EXVECTOR, exname));
+  _inputVectorLoadQueue.append(qMakePair(EYVECTOR, eyname));
   commonConstructor(in_tag, in_color);
 }
 
@@ -92,18 +96,27 @@ void KstVCurve::commonConstructor(const QString &in_tag, const QColor &in_color)
 
 
 void KstVCurve::loadInputs() {
-  VX = *KST::vectorList.findTag(_inputVectorLoadQueue[0]);
+  QValueList<QPair<QString,QString> >::Iterator i;
+  for (i = _inputVectorLoadQueue.begin(); i != _inputVectorLoadQueue.end(); ++i) {
+    if ((*i).first == XVECTOR) {
+      VX = *KST::vectorList.findTag((*i).second);
+    } else if ((*i).first == YVECTOR) {
+      VY = *KST::vectorList.findTag((*i).second);
+    } else if ((*i).first == EXVECTOR) {
+      EX = *KST::vectorList.findTag((*i).second);
+    } else if ((*i).first == EYVECTOR) {
+      EY = *KST::vectorList.findTag((*i).second);
+    }
+  }
+
   if (!VX.data()) {
-      kdWarning() << "Unable to find required vector: " << _inputVectorLoadQueue[0] << endl;
+      kdWarning() << "Unable to find required X vector!" << endl;
   }
 
-  VY = *KST::vectorList.findTag(_inputVectorLoadQueue[1]);
   if (!VY.data()) {
-      kdWarning() << "Unable to find required vector: " << _inputVectorLoadQueue[1] << endl;
+      kdWarning() << "Unable to find required Y vector!" << endl;
   }
 
-  EX = *KST::vectorList.findTag(_inputVectorLoadQueue[2]);
-  EY = *KST::vectorList.findTag(_inputVectorLoadQueue[3]);
   update();
 }
 
