@@ -22,11 +22,13 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
-#include <iostream>
 #include "kstvector.h"
 #include "kstdatacollection.h"
 #include "kstdoc.h"
 
+// Use 1 for a simple for() loop
+#define ZERO_MEMORY 2
+
 #ifdef NAN
 double KST::NOPOINT = NAN;
 #else
@@ -37,7 +39,7 @@ double KST::NOPOINT = 0.0/0.0; // NaN
 
 /** Create a vector */
 KstVector::KstVector(const QString& name, int size)
-: KstObject(), _scalars(11) {
+: KstObject(), _nsum(0), _scalars(11) {
   //kdDebug() << "+++ CREATING VECTOR: " << (void*) this << endl;
   _tag = name;
   NumShifted = 0;
@@ -63,6 +65,7 @@ KstVector::KstVector(const QString& name, int size)
   _v = static_cast<double*>(malloc(size*sizeof(double)));
   _size = size;
   zero();
+  _is_rising = false;
 
   CreateScalars();
   KST::vectorList.append(this);
@@ -187,11 +190,22 @@ void KstVector::zero() {
 }
 
 
-void KstVector::resize(int sz) {
+void KstVector::resize(int sz, bool reinit) {
   //kdDebug() << "resizing to: " << sz << endl;
-  if (sz > 0) {
-    _size = sz;
+  if (sz > 1) {
     _v = static_cast<double*>(realloc(_v, sz*sizeof(double)));
+#ifdef ZERO_MEMORY
+    if (reinit && _size < sz) {
+#if ZERO_MEMORY == 2
+      memset(&_v[_size], 0, (sz - _size)*sizeof(double));
+#else
+      for (int i = _size; i < sz; i++) {
+        _v[i] = 0.0;
+      }
+#endif
+    }
+#endif
+    _size = sz;
   }
 }
 
@@ -199,15 +213,21 @@ void KstVector::resize(int sz) {
 KstObject::UpdateType KstVector::update(int update_counter) {
   int i;
   double max, min, sum, sum2, minpos, v;
+  double last_v = -1.0E100;
 
   if (KstObject::checkUpdateCounter(update_counter))
     return NO_CHANGE;
 
   _nsum = 0;
   if (_size > 0) {
+    _is_rising = true;
+
     for (i = 0; i < _size && !finite(_v[i]); i++)
       /* Do Nothing */ ;
 
+    if (i>0)
+      _is_rising = false;
+
     max = min = sum = sum2 = _v[i];
     if (_v[i]>0)
       minpos = _v[i];
@@ -216,7 +236,13 @@ KstObject::UpdateType KstVector::update(int update_counter) {
 
     for (; i < _size; i++) {
       v = _v[i]; // get rid of redirections
+
       if (finite(v)) {
+
+        if (v<=last_v)
+          _is_rising = false;
+        last_v = v;
+
         _nsum++;
         sum += v;
         sum2 += v*v;
@@ -230,6 +256,8 @@ KstObject::UpdateType KstVector::update(int update_counter) {
             if (v>0)
               minpos = v;
         }
+      } else {
+        _is_rising = false;
       }
     }
     _scalars["sum"]->setValue(sum);
@@ -348,3 +376,4 @@ KstVectorPtr KstVector::generateVector(double x0, double x1, int n, const QStrin
 return xv;
 }
 
+
