@@ -15,22 +15,24 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <stdio.h>
+#include <assert.h>
 #include <math.h>
+#include <stdio.h>
 
 // include files for Qt
-#include <kimageio.h>
-#include <kprinter.h>
 #include <qpainter.h>
 #include <qevent.h>
 #include <qbitmap.h>
 #include <qbrush.h>
 #include <qtabwidget.h>
 
-#include <kurl.h>
-#include <ktempfile.h>
+#include <kdebug.h>
+#include <kimageio.h>
 #include <kio/netaccess.h>
 #include <klocale.h>
+#include <kprinter.h>
+#include <ktempfile.h>
+#include <kurl.h>
 
 // application specific includes
 #include "kstview.h"
@@ -40,12 +42,13 @@
 #include "kstplotdialog_i.h"
 #include "kstmouse.h"
 #include "kstlabeldialog_i.h"
-
-#include <iostream>
+#include "kstdatacollection.h"
 
 /** setup the view widget */
-KstView::KstView(KstApp *parent, const char *name) :
-  QWidget(parent, name) {
+KstView::KstView(KstApp *parent, const char *name)
+: QWidget(parent, name) {
+
+  setAcceptDrops(true);
 
   MouseInfo = new KstMouse(this);
   labelDialog = new KstLabelDialogI(parent, "Edit Label");
@@ -57,39 +60,43 @@ KstView::KstView(KstApp *parent, const char *name) :
   ZoomPlotNum = 0;
 
   ParentApp = parent;
-  needrecreate=true;
-  qpixmap=0;
+  needrecreate = true;
+  qpixmap = 0;
   setBackgroundMode(NoBackground);
   setMouseTracking(true);
 
   /* initialize the popup menu */
   popupmenu = new KPopupMenu(this);
-  popupmenu->insertItem(i18n("Zoom &Max"), this, SLOT(zoomMaxSlot()), Key_M);
-  popupmenu->insertItem(i18n("Zoom P&rev"), this, SLOT(zoomPrevSlot()), Key_R);
-  popupmenu->insertItem(i18n("Y-Zoom AC Coupled"), this, SLOT(yzoomAcSlot()), Key_A);
-  popupmenu->insertSeparator();
-  popupmenu->insertItem(i18n("X-Zoom Max"),
+  QPopupMenu *submenu;
+  submenu = new KPopupMenu(this);
+  _titleId = popupmenu->insertTitle(i18n("Plot"));
+  popupmenu->insertItem(i18n("Z&oom"), submenu);
+  submenu->insertItem(i18n("Zoom &Maximum"), this, SLOT(zoomMaxSlot()), Key_M);
+  submenu->insertItem(i18n("Zoom P&revious"), this, SLOT(zoomPrevSlot()), Key_R);
+  submenu->insertItem(i18n("Y-Zoom AC Coupled"), this, SLOT(yzoomAcSlot()), Key_A);
+  submenu->insertSeparator();
+  submenu->insertItem(i18n("X-Zoom Maximum"),
                         this, SLOT(xzoomMaxSlot()), CTRL + Key_M);
-  popupmenu->insertItem(i18n("X-Zoom Out"),
+  submenu->insertItem(i18n("X-Zoom Out"),
                         this, SLOT(xzoomOutSlot()), SHIFT + Key_Right);
-  popupmenu->insertItem(i18n("X-Zoom In"),
+  submenu->insertItem(i18n("X-Zoom In"),
                         this, SLOT(xzoomInSlot()), SHIFT + Key_Left);
-  popupmenu->insertItem(i18n("Toggle Log X Axis"),
+  submenu->insertItem(i18n("Toggle Log X Axis"),
                         this, SLOT(xLogSlot()), Key_G);
-  popupmenu->insertSeparator();
-  popupmenu->insertItem(i18n("Y-Zoom Max"),
+  submenu->insertSeparator();
+  submenu->insertItem(i18n("Y-Zoom Maximum"),
                         this, SLOT(yzoomMaxSlot()), SHIFT + Key_M);
-  popupmenu->insertItem(i18n("Y-Zoom Out"),
+  submenu->insertItem(i18n("Y-Zoom Out"),
                         this, SLOT(yzoomOutSlot()), SHIFT + Key_Up);
-  popupmenu->insertItem(i18n("Y-Zoom In"),
+  submenu->insertItem(i18n("Y-Zoom In"),
                         this, SLOT(yzoomInSlot()), SHIFT + Key_Down);
-  popupmenu->insertItem(i18n("Toggle Log Y Axis"),
-                        this, SLOT(yLogSlot()), Key_L);
-  popupmenu->insertSeparator();
-  popupmenu->insertItem(i18n("Scroll Left"), this, SLOT(leftSlot()), Key_Left);
-  popupmenu->insertItem(i18n("Scroll Right"), this, SLOT(rightSlot()), Key_Right);
-  popupmenu->insertItem(i18n("Scroll Up"), this, SLOT(upSlot()), Key_Up);
-  popupmenu->insertItem(i18n("Scroll Down"), this, SLOT(downSlot()), Key_Down);
+  submenu->insertItem(i18n("Toggle Log Y Axis"), this, SLOT(yLogSlot()), Key_L);
+  submenu = new KPopupMenu(this);
+  popupmenu->insertItem(i18n("&Scroll"), submenu);
+  submenu->insertItem(i18n("Left"), this, SLOT(leftSlot()), Key_Left);
+  submenu->insertItem(i18n("Right"), this, SLOT(rightSlot()), Key_Right);
+  submenu->insertItem(i18n("Up"), this, SLOT(upSlot()), Key_Up);
+  submenu->insertItem(i18n("Down"), this, SLOT(downSlot()), Key_Down);
 
   popupmenu->insertSeparator();
   popupmenu->insertItem(i18n("Toggle &Pause"), this, SLOT(pauseSlot()), Key_P);
@@ -99,9 +106,19 @@ KstView::KstView(KstApp *parent, const char *name) :
   popupmenu->insertSeparator();
   popupmenu->insertItem(i18n("&Edit Plots"), parent,
                         SLOT(showPlotDialog()), Key_E);
-  popupmenu->insertItem(i18n("&Data Manager"), parent,
-                        SLOT(showDataManager()), Key_D);
 
+  popupmenu->insertItem(i18n("&Delete Plot"), this, SLOT(deleteCurrentPlot()), Key_Delete);
+  popupmenu->insertItem(i18n("Clean Up Layout"), this, SLOT(cleanupLayout()));
+
+  _addMenu = new KPopupMenu(this);
+  _removeMenu = new KPopupMenu(this);
+  _editCurveMenu = new KPopupMenu(this);
+  _addMenuId = popupmenu->insertItem(i18n("&Add To Plot"), _addMenu);
+  popupmenu->setItemEnabled(_addMenuId, false);
+  _removeMenuId = popupmenu->insertItem(i18n("&Remove From Plot"), _removeMenu);
+  popupmenu->setItemEnabled(_removeMenuId, false);
+  _editCurveMenuId = popupmenu->insertItem(i18n("Edit &Curve"), _editCurveMenu);
+  popupmenu->setItemEnabled(_editCurveMenuId, false);
 
   setFocusPolicy(QWidget::StrongFocus);
   setFocus();
@@ -114,6 +131,7 @@ KstView::~KstView() {
   MouseInfo = 0L;
 }
 
+// FIXME: make this return an error code!
 void KstView::printToGraphicsFile(const QString &filename, int in_w, int in_h) {
   float w, h, tlx, tly;
   QRect v;
@@ -137,7 +155,8 @@ void KstView::printToGraphicsFile(const QString &filename, int in_w, int in_h) {
     p.setViewport((int)tlx, (int)tly, (int)w, (int)h);
     p.setWindow((int)0, (int)0, (int)w, (int)h);
     XBorder = pl->getXBorder(p);
-    if (XBorder>maxXBorder) maxXBorder = XBorder;
+    if (XBorder > maxXBorder)
+      maxXBorder = XBorder;
   }
 
   for (unsigned i = 0; i < KST::plotList.count(); i++) {
@@ -182,10 +201,17 @@ void KstView::print(KPrinter *printer)
 
   QPainter p(printer);
 
+  // Hack: if the plot is empty, at least trigger a postscript header so that
+  // we don't get a warning about x-zerosize file not being printable
+  if (KST::plotList.isEmpty()) {
+    p.moveTo(1,1);
+    return;
+  }
+
   v = p.viewport();
 
   // Find max border width
-  for (i=0; i < (int)KST::plotList.count(); i++) {
+  for (i = 0; i < (int)KST::plotList.count(); i++) {
     pl = KST::plotList.at(i);
     w = v.width() * pl->width();
     h = v.height() * pl->height();
@@ -196,10 +222,11 @@ void KstView::print(KPrinter *printer)
     h = 3000.0;
     p.setWindow((int)0, (int)0, (int)w, (int)h);
     XBorder = pl->getXBorder(p);
-    if (XBorder>maxXBorder) maxXBorder = XBorder;
+    if (XBorder > maxXBorder)
+      maxXBorder = XBorder;
   }
 
-  for (i=0; i < (int)KST::plotList.count(); i++) {
+  for (i = 0; i < (int)KST::plotList.count(); i++) {
     pl = KST::plotList.at(i);
     w = v.width() * pl->width();
     h = v.height() * pl->height();
@@ -238,7 +265,7 @@ void KstView::paintEvent(QPaintEvent *) {
     i1 = 0;
   } else if (ZoomCurrentPlot) {
     i0 = ZoomPlotNum;
-    i1 = i0+1;
+    i1 = i0 + 1;
   } else {
     i0 = 0;
     i1 = n_plots;
@@ -249,7 +276,7 @@ void KstView::paintEvent(QPaintEvent *) {
     qpixmap->fill();
 
     // Find max border width
-    for (i=i0; i < i1; i++) {
+    for (i = i0; i < i1; i++) {
       /* Draw each plot to its own pixmap */
       pl = KST::plotList.at(i);
       if (ZoomCurrentPlot) {
@@ -263,24 +290,25 @@ void KstView::paintEvent(QPaintEvent *) {
         tly = height() * pl->topLeftY();
         tlx = width() * pl->topLeftX();
       }
-      if (i0==i1) { // only one plot - draw directly onto qpixmap
+      if (i0 == i1) { // only one plot - draw directly onto qpixmap
         tmpPixmap = qpixmap;
         p = new QPainter(tmpPixmap);
-      } else if (i==i0) {
+      } else if (i == i0) {
         tmpPixmap = new QPixmap(int(w),int(h));
         tmpPixmap->setOptimization(QPixmap::MemoryOptim);
         p = new QPainter(tmpPixmap);
-      } else if ((tmpPixmap->width() != w) || (tmpPixmap->height() != h)) {
+      } else if (tmpPixmap->width() != w || tmpPixmap->height() != h) {
         delete p;
         tmpPixmap->resize(int(w),int(h));
         p = new QPainter(tmpPixmap);
       }
 
       XBorder = pl->getXBorder(*p);
-      if (XBorder>maxXBorder) maxXBorder = XBorder;
+      if (XBorder > maxXBorder)
+        maxXBorder = XBorder;
     }
 
-    for (i=i0; i < i1; i++) {
+    for (i = i0; i < i1; i++) {
       /* Draw each plot to its own pixmap */
       pl = KST::plotList.at(i);
       if (ZoomCurrentPlot) {
@@ -294,46 +322,45 @@ void KstView::paintEvent(QPaintEvent *) {
         tly = height() * pl->topLeftY();
         tlx = width() * pl->topLeftX();
       }
-      if ((tmpPixmap->width() != w) || (tmpPixmap->height() != h)) {
+      if (tmpPixmap->width() != w || tmpPixmap->height() != h) {
         delete p;
         tmpPixmap->resize((int)w,(int)h);
         p = new QPainter(tmpPixmap);
       }
       pl->paint(*p, maxXBorder);
       /* copy the tmppix map to the qpixmap */
-      if (i0!=i1) {
-        bitBlt(qpixmap, (int)tlx, (int)tly, tmpPixmap,0,0,-1,-1,
+      if (i0 != i1) {
+        bitBlt(qpixmap, (int)tlx, (int)tly, tmpPixmap, 0, 0, -1, -1,
                QWidget::CopyROP, true);
       }
       pl->setpixrect((int)tlx, (int)tly);
     }
     delete p;
-    if (tmpPixmap!=qpixmap)
+    if (tmpPixmap != qpixmap)
       delete tmpPixmap;
   }
 
-  bitBlt(this,0,0,qpixmap,0,0,-1,-1,
-         QWidget::CopyROP, true);
+  bitBlt(this, 0, 0, qpixmap, 0, 0, -1, -1, QWidget::CopyROP, true);
   updateTieBoxes(this);
 
   needrecreate = false;
 
   // Redraw the mouse box if needed
-  if (MouseInfo->getMode()!=INACTIVE) {
+  if (MouseInfo->getMode() != INACTIVE) {
     QPainter p(this);
-    if (!MouseInfo->isFirstMove()) MouseInfo->DrawBox(p);
+    if (!MouseInfo->isFirstMove())
+      MouseInfo->DrawBox(p);
   }
 }
 
 void KstView::resizeEvent(QResizeEvent *) {
-  needrecreate=true;
+  needrecreate = true;
 }
 
 
 void KstView::update() {
-  needrecreate=true;
+  needrecreate = true;
   paintEvent(NULL);
-
 }
 
 void KstView::mouseMoveEvent(QMouseEvent *e) {
@@ -343,41 +370,75 @@ void KstView::mouseMoveEvent(QMouseEvent *e) {
   QRect plot_rect;
   double xmin, xmax, ymin, ymax, xpos, ypos;
   bool in_plot = false;
-  int plot_num;
+  int i_label;
+  int plot_num = MouseInfo->getPlotNum();
 
-  plot_num = MouseInfo->getPlotNum();
+  if (e->state() & Qt::LeftButton &&
+        MouseInfo->getMode() == LABEL_TOOL &&
+        (i_label = MouseInfo->getLabelNum(e)) >= 0 &&
+        _draggableLabel == i_label &&
+        plot_num == _draggablePlot) {
+    // Start a drag
+    KstPlot *plot = KST::plotList.at(plot_num);
+    KstLabel *label = plot->labelList.at(i_label);
+    QRect oldExtents = label->extents;
+    QPixmap pm(plot->GetWinRegion().width(), plot->GetWinRegion().height());
+    pm.fill();
+    { // Scope is needed to kill off the painter before we resize
+      QPainter p(&pm);
+      label->draw(p, 2, oldExtents.height() + 2, true);
+    }
+    pm.resize(oldExtents.width() + 4, oldExtents.height() + 4);
+    label->extents = oldExtents; // restore them incase the drag is canceled
+    QDragObject *d = new KstLabelDrag(this, plot_num, i_label, _draggablePoint - plot->GetWinRegion().topLeft() - oldExtents.topLeft(), pm);
+    d->dragMove();
+    _draggableLabel = -1;
+    _draggablePlot = -1;
+    return;
+  } else if (MouseInfo->getMode() == XY_ZOOMBOX) {
+    QRect pr = KST::plotList.at(plot_num)->GetPlotRegion();
 
-  if (MouseInfo->getMode() == XY_ZOOMBOX) {
-    QRect pr = (KST::plotList.at(plot_num))->
-               GetPlotRegion();
+    if (e->x() > pr.right()) {
+      x = pr.right() + 1;
+    } else if (e->x() < pr.left()) {
+      x = pr.left();
+    } else {
+      x = e->x();
+    }
 
-    if (e->x() > pr.right()) x = pr.right()+1;
-    else if (e->x() < pr.left()) x = pr.left();
-    else x = e->x();
-
-    if (e->y() > pr.bottom()) y = pr.bottom()+1;
-    else if (e->y() < pr.top()) y = pr.top();
-    else y = e->y();
+    if (e->y() > pr.bottom()) {
+      y = pr.bottom() + 1;
+    } else if (e->y() < pr.top()) {
+      y = pr.top();
+    } else {
+      y = e->y();
+    }
 
     MouseInfo->setLastLocation(QPoint(x,y));
   } else if (MouseInfo->getMode() == Y_ZOOMBOX) {
-    QRect pr = (KST::plotList.at(plot_num))->
-               GetPlotRegion();
+    QRect pr = KST::plotList.at(plot_num)->GetPlotRegion();
 
     x = pr.right();
 
-    if (e->y() > pr.bottom()) y = pr.bottom()+1;
-    else if (e->y() < pr.top()) y = pr.top();
-    else y = e->y();
+    if (e->y() > pr.bottom()) {
+      y = pr.bottom() + 1;
+    } else if (e->y() < pr.top()) {
+      y = pr.top();
+    } else {
+      y = e->y();
+    }
 
     MouseInfo->setLastLocation(QPoint(x,y));
   } else if (MouseInfo->getMode() == X_ZOOMBOX) {
-    QRect pr = (KST::plotList.at(plot_num))->
-               GetPlotRegion();
+    QRect pr = KST::plotList.at(plot_num)->GetPlotRegion();
 
-    if (e->x() > pr.right()) x = pr.right()+1;
-    else if (e->x() < pr.left()) x = pr.left();
-    else x = e->x();
+    if (e->x() > pr.right()) {
+      x = pr.right() + 1;
+    } else if (e->x() < pr.left()) {
+      x = pr.left();
+    } else {
+      x = e->x();
+    }
 
     y = pr.bottom();
 
@@ -387,28 +448,33 @@ void KstView::mouseMoveEvent(QMouseEvent *e) {
   // change the xy field
   //emit newDataMsg(msg);
 
-  msg = "";
+  msg = QString::null;
 
-  for (i=0; i < (int)KST::plotList.count(); i++) {
-    plot_rect = (KST::plotList.at(i))->GetPlotRegion();
+  for (i = 0; i < (int)KST::plotList.count(); i++) {
+    plot_rect = KST::plotList.at(i)->GetPlotRegion();
     if (plot_rect.contains(e->pos())) {
-      (KST::plotList.at(i))->getLScale(xmin, ymin, xmax, ymax);
+      KST::plotList.at(i)->getLScale(xmin, ymin, xmax, ymax);
       xpos = (double)(e->x() - plot_rect.left())/(double)plot_rect.width();
       xpos = xpos * (xmax - xmin) + xmin;
       ypos = (double)(e->y() - plot_rect.top())/(double)plot_rect.height();
       ypos = ypos * (ymin-ymax) + ymax;
+
       if (KST::plotList.at(i)->isXLog()) {
         xpos = pow(10.0, xpos);
       }
+
       if (KST::plotList.at(i)->isYLog()) {
         ypos = pow(10.0, ypos);
       }
+
       msg = i18n("(%1, %2)").arg(xpos,0,'G').arg(ypos,0,'G');
       if (MouseInfo->getMode() == INACTIVE) {
-        if (i!=plot_num) {
+        if (i != plot_num) {
           MouseInfo->setPlotNum(i);
           updateTieBoxes(this);
         }
+      } else if (MouseInfo->getMode() == LABEL_TOOL) {
+        MouseInfo->setPlotNum(i);
       }
       in_plot = true;
     }
@@ -425,6 +491,9 @@ void KstView::mousePressEvent(QMouseEvent *e) {
   /* Find where the mouse was to determine which mode to be in */
   /* which button */
   if (e->button() == Qt::LeftButton) {
+    _draggablePlot = -1;
+    _draggableLabel = MouseInfo->getLabelNum(e);
+    _draggablePoint = e->pos();
     /* search through the plots */
     for (i = 0; i < KST::plotList.count(); i++) {
       plot = KST::plotList.at(i);
@@ -435,36 +504,73 @@ void KstView::mousePressEvent(QMouseEvent *e) {
       if (tie_rect.contains(e->pos())) {
         plot->toggleTied();
         updateTieBoxes(this);
+        break;
       } else if (plot_rect.contains(e->pos())) {
-        MouseInfo->mousePressedInPlot(e, plot_rect);
+        if (_draggableLabel == -1 || MouseInfo->getMode() != LABEL_TOOL) {
+          MouseInfo->mousePressedInPlot(e, plot_rect);
+        } else {
+          _draggablePlot = i;
+        }
         MouseInfo->setPlotNum(i);
         return;
       } else if (plot_and_axis_rect.contains(e->pos())) {
         MouseInfo->setPlotNum(i);
-        ParentApp->plotDialog->Select->setCurrentItem(i);
         ParentApp->plotDialog->show_I();
+        ParentApp->plotDialog->Select->setCurrentItem(i);
+        ParentApp->plotDialog->update();
         ParentApp->plotDialog->TabWidget->setCurrentPage(LIMITS_TAB);
+        break;
       } else if (win_rect.contains(e->pos())) {
         MouseInfo->setPlotNum(i);
-        ParentApp->plotDialog->Select->setCurrentItem(i);
         ParentApp->plotDialog->show_I();
+        ParentApp->plotDialog->Select->setCurrentItem(i);
+        ParentApp->plotDialog->update();
         ParentApp->plotDialog->TabWidget->setCurrentPage(LABELS_TAB);
+        break;
       }
     }
   } else if (e->button() == Qt::RightButton) {
     for (i = 0; i < KST::plotList.count(); i++) {
-      win_rect = KST::plotList.at(i)->GetPlotRegion();
+      KstPlot *p = KST::plotList.at(i);
+      win_rect = p->GetPlotRegion();
       if (win_rect.contains(e->pos())) {
         MouseInfo->setMode(INACTIVE);
         MouseInfo->setPressLocation(e->pos());
         MouseInfo->setPlotNum(i);
+        popupmenu->changeTitle(_titleId, i18n("Plot: %1").arg(p->tagName()));
+
+        // Update the add/remove menus.
+        _addMenu->clear();
+        _removeMenu->clear();
+        _editCurveMenu->clear();
+        _currentPlot = p;
+        int id = 100;
+        bool addItemEnabled = false, removeItemEnabled = false;
+        KstBaseCurveList curves = kstObjectSubList<KstDataObject, KstBaseCurve>(KST::dataObjectList);
+        for (KstBaseCurveList::Iterator i = curves.begin(); i != curves.end(); ++i) {
+          if (!p->Curves.contains((*i).data())) {
+            _addMenu->insertItem((*i)->tagName(), this, SLOT(dynamicMenuAdd(int)), 0, id);
+            addItemEnabled = true;
+          } else {
+            _removeMenu->insertItem((*i)->tagName(), this, SLOT(dynamicMenuRemove(int)), 0, id);
+            removeItemEnabled = true;
+            _editCurveMenu->insertItem((*i)->tagName(), this, SLOT(dynamicMenuEdit(int)), 0, id);
+          }
+          _curveIds[id] = (*i)->tagName();
+          ++id;
+        }
+
+        popupmenu->setItemEnabled(_addMenuId, addItemEnabled);
+        popupmenu->setItemEnabled(_removeMenuId, removeItemEnabled);
+        popupmenu->setItemEnabled(_editCurveMenuId, removeItemEnabled);
+
         popupmenu->popup(mapToGlobal(e->pos()));
         return;
       }
     }
   } else if (e->button() == Qt::MidButton) {
-    for (i=0; i < KST::plotList.count(); i++) {
-      win_rect = (KST::plotList.at(i))->GetPlotRegion();
+    for (i = 0; i < KST::plotList.count(); i++) {
+      win_rect = KST::plotList.at(i)->GetPlotRegion();
       if (win_rect.contains(e->pos())) {
         MouseInfo->setMode(INACTIVE);
         MouseInfo->setPressLocation(e->pos());
@@ -483,15 +589,17 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
   double xmin, xmax, ymin, ymax;
   double new_xmin, new_xmax, new_ymin, new_ymax;
   QRect plotregion;
-  unsigned int i_plot;
   KstPlot *plot, *iplot;
   unsigned int count;
 
+  _draggableLabel = -1;
+  _draggablePlot = -1;
+
   if (KST::plotList.isEmpty()) {
     return;
   }
 
-  i_plot = MouseInfo->getPlotNum();
+  unsigned int i_plot = MouseInfo->getPlotNum();
   plot = KST::plotList.at(i_plot);
 
   if (MouseInfo->getMode() == XY_ZOOMBOX) {
@@ -577,31 +685,82 @@ void KstView::mouseReleaseEvent(QMouseEvent *e) {
       }
     }
   } else if (MouseInfo->getMode() == LABEL_TOOL) {
-    int i_label;
-    i_label = MouseInfo->getLabelNum(e);
+    int i_label = MouseInfo->getLabelNum(e);
 
     plotregion = plot->GetPlotRegion();
-    double x = (double)(e->x() - plotregion.left())/
-               (double)(plotregion.width());
-    double y = (double)(e->y() - plotregion.top())/
-               (double)(plotregion.height());
-    labelDialog->showI(i_plot, i_label, x, y);
+    if (i_label >= 0 || plotregion.contains(e->pos()))  {
+      double x = double(e->x() - plotregion.left()) / double(plotregion.width());
+      double y = double(e->y() - plotregion.top()) / double(plotregion.height());
+      labelDialog->showI(i_plot, i_label, x, y);
+    }
   }
 
   MouseInfo->setMode(INACTIVE);
   needrecreate = true;
   update();
+
+  // Why?
   mouseMoveEvent(e);
 }
 
-void KstView::zoomMaxSlot() {
-  unsigned int i_plot, count;
-  KstPlot *iplot;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
-    iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+void KstView::dragEnterEvent(QDragEnterEvent *e) {
+  e->accept(e->provides("application/x-kst-label-number"));
+}
+
+
+void KstView::dropEvent(QDropEvent *e) {
+  if (e->provides("application/x-kst-label-number")) {
+    if (e->source() == this) {
+      QDataStream ds(e->encodedData("application/x-kst-label-number"), IO_ReadOnly);
+      int plot, label;
+      QPoint hs;
+      ds >> plot >> label >> hs;
+      int posplot = -1, plotCount = KST::plotList.count();
+      KstPlot *plotPtr = 0L;
+      for (int i = 0; i < plotCount; i++) {
+        plotPtr = KST::plotList.at(i);
+        if (plotPtr->GetPlotRegion().contains(e->pos())) {
+          posplot = i;
+          break;
+        }
+      }
+
+      if (posplot == -1) {
+        e->accept(false);
+        return;
+      }
+
+      KstLabel *l;
+      if (posplot != plot) { // move to a new plot
+        l = KST::plotList.at(plot)->labelList.take(label);
+        assert(l);
+        plotPtr->labelList.append(l);
+      } else {
+        l = plotPtr->labelList.at(label);
+      }
+
+      QPoint pos = e->pos() - plotPtr->GetPlotRegion().topLeft() - hs + QPoint(2 /* border offset */, 2 /* border offset */ + 8 /* fudge factor */);
+      QSize divisor = plotPtr->GetPlotRegion().size();
+      l->move(float(pos.x())/divisor.width(), float(pos.y())/divisor.height());
+      l->extents.setX(pos.x());
+      l->extents.setY(pos.y());
+      e->accept(true);
+      update();
+      return;
+    }
+  }
+
+  e->accept(false);
+}
+
+
+void KstView::zoomMaxSlot() {
+  unsigned int count = KST::plotList.count();
+
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
+    KstPlot *iplot = KST::plotList.at(i_plot);
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setXScaleMode(AUTO);
       iplot->setYScaleMode(AUTO);
       iplot->pushScale();
@@ -610,7 +769,6 @@ void KstView::zoomMaxSlot() {
   MouseInfo->setMode(INACTIVE);
   needrecreate = true;
   update();
-
 }
 
 void KstView::keyPressEvent(QKeyEvent *e) {
@@ -629,13 +787,12 @@ void KstView::keyReleaseEvent(QKeyEvent *e) {
 }
 
 void KstView::xzoomMaxSlot() {
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *iplot;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setXScaleMode(AUTO);
       iplot->pushScale();
     }
@@ -646,13 +803,12 @@ void KstView::xzoomMaxSlot() {
 }
 
 void KstView::yzoomMaxSlot() {
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *iplot;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setYScaleMode(AUTO);
       iplot->pushScale();
     }
@@ -663,13 +819,12 @@ void KstView::yzoomMaxSlot() {
 }
 
 void KstView::yzoomAcSlot() {
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *iplot;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setYScaleMode(AC);
       iplot->pushScale();
     }
@@ -682,10 +837,10 @@ void KstView::yzoomAcSlot() {
 void KstView::zoomOutSlot() {
   double xmin, xmax, ymin, ymax;
   double new_xmin, new_xmax, new_ymin, new_ymax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_xmin = xmin - (xmax - xmin)*0.25;
@@ -693,10 +848,9 @@ void KstView::zoomOutSlot() {
   new_ymin = ymin - (ymax - ymin)*0.25;
   new_ymax = ymax + (ymax - ymin)*0.25;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setXScaleMode(FIXED);
       iplot->setYScaleMode(FIXED);
       iplot->setLScale(new_xmin, new_ymin, new_xmax, new_ymax);
@@ -712,19 +866,18 @@ void KstView::zoomOutSlot() {
 void KstView::xzoomInSlot() {
   double xmin, xmax, ymin, ymax;
   double new_xmin, new_xmax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_xmin = xmin + (xmax - xmin)*0.1666666;
   new_xmax = xmax - (xmax - xmin)*0.1666666;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setXScaleMode(FIXED);
       iplot->setLXScale(new_xmin, new_xmax);
       iplot->pushScale();
@@ -737,19 +890,18 @@ void KstView::xzoomInSlot() {
 void KstView::xzoomOutSlot() {
   double xmin, xmax, ymin, ymax;
   double new_xmin, new_xmax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_xmin = xmin - (xmax - xmin)*0.25;
   new_xmax = xmax + (xmax - xmin)*0.25;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setXScaleMode(FIXED);
       iplot->setLXScale(new_xmin, new_xmax);
       iplot->pushScale();
@@ -764,19 +916,18 @@ void KstView::xzoomOutSlot() {
 void KstView::yzoomInSlot() {
   double xmin, xmax, ymin, ymax;
   double new_ymin, new_ymax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_ymin = ymin + (ymax - ymin)*0.16666666;
   new_ymax = ymax - (ymax - ymin)*0.16666666;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setYScaleMode(FIXED);
       iplot->setLYScale(new_ymin, new_ymax);
       iplot->pushScale();
@@ -790,19 +941,18 @@ void KstView::yzoomInSlot() {
 void KstView::yzoomOutSlot() {
   double xmin, xmax, ymin, ymax;
   double new_ymin, new_ymax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_ymin = ymin - (ymax - ymin)*0.25;
   new_ymax = ymax + (ymax - ymin)*0.25;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setYScaleMode(FIXED);
       iplot->setLYScale(new_ymin, new_ymax);
       iplot->pushScale();
@@ -814,15 +964,14 @@ void KstView::yzoomOutSlot() {
 }
 
 void KstView::zoomPrevSlot() {
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   bool popped = false;
   KstPlot *plot;
 
   /* pop prev zoom on all tied plots */
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     plot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (plot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || plot->isTied()) {
       if (plot->popScale()) {
         popped = true;
       }
@@ -839,19 +988,18 @@ void KstView::zoomPrevSlot() {
 void KstView::upSlot() {
   double xmin, xmax, ymin, ymax;
   double new_ymin, new_ymax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_ymin = ymin + (ymax - ymin)*0.25;
   new_ymax = ymax + (ymax - ymin)*0.25;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->getLScale(xmin, ymin, xmax, ymax);
       new_ymin = ymin + (ymax - ymin)*0.25;
       new_ymax = ymax + (ymax - ymin)*0.25;
@@ -867,22 +1015,21 @@ void KstView::upSlot() {
 }
 
 void KstView::downSlot() {
-    double xmin, xmax, ymin, ymax;
+  double xmin, xmax, ymin, ymax;
   double new_ymin, new_ymax;
-  unsigned int i_plot,count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_ymin = ymin - (ymax - ymin)*0.25;
   new_ymax = ymax - (ymax - ymin)*0.25;
 
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       if (ParentApp->getMouseZoomRadio() == X_ZOOMBOX) {
         iplot->getLScale(xmin, ymin, xmax, ymax);
         new_ymin = ymin - (ymax - ymin)*0.25;
@@ -904,19 +1051,18 @@ void KstView::downSlot() {
 void KstView::rightSlot() {
   double xmin, xmax, ymin, ymax;
   double new_xmin, new_xmax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_xmin = xmin + (xmax - xmin)*0.1;
   new_xmax = xmax + (xmax - xmin)*0.1;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setXScaleMode(FIXED);
       iplot->setLXScale(new_xmin, new_xmax);
       iplot->pushScale();
@@ -930,19 +1076,18 @@ void KstView::rightSlot() {
 void KstView::leftSlot() {
   double xmin, xmax, ymin, ymax;
   double new_xmin, new_xmax;
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   plot->getLScale(xmin, ymin, xmax, ymax);
 
   new_xmin = xmin - (xmax - xmin)*0.1;
   new_xmax = xmax - (xmax - xmin)*0.1;
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setXScaleMode(FIXED);
       iplot->setLXScale(new_xmin, new_xmax);
       iplot->pushScale();
@@ -958,18 +1103,17 @@ void KstView::pauseSlot(){
 }
 
 void KstView::xLogSlot() {
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
   bool xLog, yLog;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   xLog = !plot->isXLog();
   yLog = plot->isYLog();
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setLog(xLog, yLog);
     }
   }
@@ -979,18 +1123,17 @@ void KstView::xLogSlot() {
 }
 
 void KstView::yLogSlot() {
-  unsigned int i_plot, count;
+  unsigned int count = KST::plotList.count();
   KstPlot *plot, *iplot;
   bool xLog, yLog;
 
-  plot = (KST::plotList.at(MouseInfo->getPlotNum()));
+  plot = KST::plotList.at(MouseInfo->getPlotNum());
   xLog = plot->isXLog();
   yLog = !plot->isYLog();
 
-  count = KST::plotList.count();
-  for (i_plot=0; i_plot<count; i_plot++) {
+  for (unsigned i_plot = 0; i_plot < count; i_plot++) {
     iplot = KST::plotList.at(i_plot);
-    if ((i_plot==MouseInfo->getPlotNum()) || (iplot->isTied())) {
+    if (i_plot == MouseInfo->getPlotNum() || iplot->isTied()) {
       iplot->setLog(xLog, yLog);
     }
   }
@@ -1022,7 +1165,7 @@ void KstView::updateTieBox(QPaintDevice *pd, unsigned int i) {
     p.fillRect(tie_box, QColor("white"));
   }
   p.drawRoundRect(tie_box);
-  if (i==MouseInfo->getPlotNum()) {
+  if (i == MouseInfo->getPlotNum()) {
     p.fillRect(s_left, s_top, s_dim, s_dim, QColor("black"));
   } else {
     p.fillRect(s_left, s_top, s_dim, s_dim, QColor("white"));
@@ -1030,32 +1173,31 @@ void KstView::updateTieBox(QPaintDevice *pd, unsigned int i) {
 }
 
 void KstView::updateTieBoxes(QPaintDevice *pd) {
-  unsigned int i, count;
+  unsigned int count = KST::plotList.count();
 
   if (ZoomCurrentPlot) return;
 
-  count = KST::plotList.count();
-  for (i=0; i < count; i++) {
+  for (unsigned i = 0; i < count; i++) {
     updateTieBox(pd, i);
   }
 }
 
 void KstView::toggleTiedZoom() {
-  int i, count;
+  unsigned count = KST::plotList.count();
   KstPlot *plot;
-  int num_tied=0;
+  unsigned num_tied = 0;
   bool new_tie_state;
 
   // the plots get to vote on whether we are going to tie or untie...
-  count = (int)KST::plotList.count();
-  for (i=0; i < count; i++) {
+  for (unsigned i = 0; i < count; i++) {
     plot = KST::plotList.at(i);
-    if (plot->isTied()) num_tied++;
+    if (plot->isTied())
+      num_tied++;
   }
-  if (num_tied>count/2) new_tie_state = false;
-  else new_tie_state = true;
 
-  for (i=0; i < count; i++) {
+  new_tie_state = num_tied <= count/2;
+
+  for (unsigned i = 0; i < count; i++) {
     plot = KST::plotList.at(i);
     plot->setTied(new_tie_state);
   }
@@ -1065,10 +1207,13 @@ void KstView::toggleTiedZoom() {
 void KstView::zoomPlotSlot() {
   ZoomCurrentPlot = !ZoomCurrentPlot;
 
-  if (ZoomCurrentPlot) MouseInfo->setZoomed();
-  else MouseInfo->unsetZoom();
+  if (ZoomCurrentPlot) {
+    MouseInfo->setZoomed();
+  } else {
+    MouseInfo->unsetZoom();
+  }
 
-  ZoomPlotNum =  MouseInfo->getPlotNum();
+  ZoomPlotNum = MouseInfo->getPlotNum();
   update();
 }
 
@@ -1077,4 +1222,61 @@ void KstView::forceUpdate() {
   paintEvent(0L);
 }
 
+void KstView::dynamicMenuAdd(int id) {
+   dynamicMenu(true, id);
+}
+
+void KstView::dynamicMenuRemove(int id) {
+   dynamicMenu(false, id);
+}
+
+void KstView::dynamicMenuEdit(int id) {
+  KstDataObjectList::Iterator i = KST::dataObjectList.findTag(_curveIds[id]);
+  if (i == KST::dataObjectList.end()) {
+    return;
+  }
+  (*i)->showDialog();
+
+  _currentPlot = 0L;
+}
+
+void KstView::dynamicMenu(bool add, int id) {
+  if (!_currentPlot) {
+    return;
+  }
+
+  KstDataObjectList::Iterator i = KST::dataObjectList.findTag(_curveIds[id]);
+  KstBaseCurve *di = 0;
+  if (i == KST::dataObjectList.end() || !(di = dynamic_cast<KstBaseCurve*>((*i).data()))) {
+    return;
+  }
+  KstBaseCurvePtr p(di);
+  if (add) {
+    if (!_currentPlot->Curves.contains(p)) {
+      _currentPlot->addCurve(p);
+      update();
+    }
+  } else {
+    _currentPlot->removeCurve(p);
+    update();
+  }
+
+  _currentPlot = 0L;
+}
+
+void KstView::deleteCurrentPlot() {
+  if (_currentPlot) {
+    KST::plotList.remove(_currentPlot);
+    _currentPlot = 0L;
+    update();
+  }
+}
+
+void KstView::cleanupLayout() {
+  needrecreate = true;
+  KST::plotList.arrangePlots(KST::plotList.getPlotCols());
+  update();
+}
+
 #include "kstview.moc"
+// vim: ts=2 sw=2 et
